function [boomsize,bustsize,reboundsize,bbrgrowth,boomlength,bustlength,maxforeclosure,std_dP_bust,kurtosis_dP_bust,maxdprice_bust,maxdprice_rebound] = ...
    calc_moments(T,P,fore_rate)
    % Returns moments from time series
    % Boom size: Initial to peak log difference
    % Bust size: Initial to trough log difference
    % Boom length: Initial to peak in years
    % Bust length: Peak to trough in years
    % Max Foreclosure: Peak of Foreclosures 1 year MA
    % Mean Foreclosure: Mean of Foreclosures from peak to peak + 7
    % Const_boom: Initial SS to 1 year MA around price peak log diff in
    % construction
    % Const_bust: 1 year MA around price peak minus one year MA around
    % price trough log dif in construction
    
    % calculate price peak
    P_MA = movmean(P,13);
    [val pos] = min(diff(P_MA));
    [maxval,maxper] = max(P_MA(1:min(pos+52,length(P_MA))));
    [minval,minper] = min(P_MA(maxper:end));
    
    peakdate = maxper;
    troughdate = maxper+minper-1;
    
    % Calculate boom and bust size and length from weekly data
    maxT = T(peakdate);
    minT = T(troughdate);
    boomlength = maxT;
    bustlength = minT-maxT;
    
    % Calculate boom size and bust size from quarterly data.
    P_MA_q = P_MA([7:13:length(T)]);
    [val pos] = min(diff(P_MA_q));
    [maxval,maxper] = max(P_MA_q(1:min(pos+13,length(P_MA_q))));
    [minval,minper] = min(P_MA_q(maxper:end));
    
    peakdate_q = maxper;
    troughdate_q = maxper+minper-1;
    
    boomsize = log(maxval) - log(P(1)); % Measure boomsize based on old steady state
    bustsize = log(maxval) - log(minval);
    rebound_enddate = min(length(P_MA_q),troughdate_q + 7*4);
    reboundsize = log(P_MA_q(rebound_enddate)) - log(P_MA_q(troughdate_q));
    
    % Calculate foreclosure moments
    Fore_MA = movmean(fore_rate,13); %3 month MA in data.
    maxforeclosure = max(Fore_MA);
    
    % Calculate bust std 4q price changes and kurtosis 4q price changes
    dP_MA = log(P_MA_q(5:end)) - log(P_MA_q(1:(end-4)));
    dP_MA_q = log(P_MA_q(5:end)) - log(P_MA_q(1:(end-4)));
    std_dP_bust = std(dP_MA(peakdate_q:(troughdate_q-4)));
    kurtosis_dP_bust = kurtosis(dP_MA(peakdate_q:(troughdate_q-4)));
    if(isempty(std_dP_bust)) 
        std_dP_bust = 0;
        kurtosis_dP_bust = 0;
    end
    
    % max 1 year price change bust
    maxdprice_bust = min(dP_MA_q(peakdate_q:(troughdate_q-4)));
    if(isempty(maxdprice_bust)) 
        maxdprice_bust = 0;
    end
    
    % max 1 year price change rebound
    maxdprice_rebound = max(dP_MA_q((troughdate_q-4):min(troughdate_q+6*4,end)));
    if(isempty(maxdprice_rebound)) 
        maxdprice_rebound = 0;
    end
    
    % BBR GRowth is 1998Q1 to 2019Q
    sampleend = 88; %2019q4
    boomstartcommon = 1; % 1998q1
    lhpi_end = log(P_MA_q(sampleend));
    lhpi_start = log(P_MA_q(boomstartcommon));
    bbrgrowth = (lhpi_end - lhpi_start)/((sampleend-(boomstartcommon))/4);
    
    %{
    %Debug plot price peak and trough.
    subplot(3,1,1), plot(T,P_MA);
    hold on;
    scatter(maxT,maxval,25,'red','filled');
    scatter(minT,minval,25,'blue','filled');
    hold off;
    
    subplot(3,1,2), plot(T,Fore_MA)
    hold on;
    scatter(T(maxforeclosureper),maxforeclosure,'red','filled');
    plot([maxT,maxT+7],[meanforeclosure,meanforeclosure]);
    hold off;
    
    subplot(3,1,3), plot(T,const_MA);
    hold on;
    scatter(maxT,const_MA(peakdate),25,'red','filled');
    scatter(minT,const_MA(troughdate),25,'blue','filled');
    hold off;
    %}
end

