clear;
addpath(genpath('/projectnb/gurengrp2/CRGM/SMM/shared_libraries'))
addpath(genpath('/Users/adam/Dropbox/Research/House Prices Growth Misperception (CRG)/Code'))
addpath(genpath('../'))

% exogenous params
    
eta=1/0.63;
gamma_elas=0.5;
gam=0;
iota_tot = .045;
theta=0.005; %Mean reversion in mu: set exogenously
mubar=0.004; %Pre-boom and long-run mean drift rate: set exogenously
sigma_mu=.001; %Standard deviation of drift rate Wiener: set exogenously
LTV_load_flag='on';
save('exog_params','gamma_elas','gam','iota_tot','theta','mubar','sigma_mu','LTV_load_flag');

mu0=.035; %Drift rate after "jump" at date 0: SMM grid
%mu0=mubar; %Drift rate after "jump" at date 0: SMM grid
kappa=.2; %Ratio of Kalman gain to standard deviation of dividend Wiener: SMM grid
nu=.05; %Decay of new information --- controls length of the boom
phi=2.75; %Diagnostic over-weighting --- Bordalo et al advocate for value of 3: SMM grid
rho=.07; %Discount rate: SMM grid
iota_L = .045;
chi = .04;
bb = .15;
aa = 1;
iota_R = max(iota_tot - iota_L,0);
k=9;


%load('../../SMM/Diagnostic/optimal_params.mat');

    
% nTrials

load exog_params;
    
    save('passed_params','chi','gamma_elas','eta','gam','mu0','mubar','sigma_mu','theta',...
        'nu','k','kappa','phi','iota_L','iota_R','aa','bb','rho','LTV_load_flag');

    %% Set up paramaters

addpath(genpath(pwd));
run gen_parameters_passparams;

nTrials_mc=100;
nTrials_irf=2000;
    
nTrials=int16(nTrials_mc);%Number of trials for Monte-Carlo
T=30; %Years
ff=365; %Step-size
tol=1e-3; %Stopping criteria
max_its=50; %Max number of iterations
w=3; %Level of the Smolyak grid
approx_flag='linear'; %Use linear approximation ('linear') of polynomial interpolation ('global')

close all;
load params
iota_R=iota_R/ff;
iota_L=iota_L/ff;
% iota=0;
gamma=gamma_elas;
alpha=alpha_Ibar;
mu=mu_g;
sigma=sigma_D;




%% Create sparse grid

%Create a Smolyak sparse grid.  Uses Crenshaw-Curtis nested points, where w
%denotes the level of the sparse grid.
chebsize=0;
[S,Sr,n,domain,D_grid,m_grid,H_grid,Ibar_grid]...
    = grid_construct(w,chebsize);

%Solve for the equilibrium price function on the sparse grid and create a
%best linear approximation

V_grid=PV(m_grid,rho,mubar,theta,sigma_mu,sigma_m,D_grid);
W=zeros(1,n);
options = optimoptions('fsolve','SpecifyObjectiveGradient',true,'Display','off');
[P0,I0,flag] = Solve_Eq_Price_Global(V_grid,H_grid,gH,c,eta,chi,xm,gamma,Ibar_grid,fore_rate_BGP,W,n,V0,options);

Xgrid=[D_grid' m_grid' H_grid' Ibar_grid' ones(n,1)];
  
[bP,bI,~,R2] = ConstructApprox(P0,I0,Xgrid,S,Sr,domain,approx_flag);
P0_predict=Xgrid*bP;
scatter(P0_predict,P0);
        

%% Set up the Monto-Carlo simulation

dt=1/ff; %Simulation stepsize
nPeriods=T/dt; %Number of simulation periods
nBrowns=2;
bet=exp(-rho*dt); 
r=1/bet-1; %Implied interest rate
n=uint16(n);
fore_disc=.74;

load Zsave;
if ~isequal(size(Z),[nPeriods,nBrowns,nTrials])
        Z=genZ(1,nBrowns,nPeriods,nTrials,dt);
        save('Zsave','Z');
end

tic
[W,W_mat] = MC_loop_parallel(bP,bI,I0,P0,Xgrid,T,dt,alpha,aa,bb,iota_L,iota_R,sigma,...
    theta,mubar,K,r,LTV,n,nTrials,Z,fore_disc,gam);
toc