% clear;
load('passed_params');

% Now load other params not passed

%% Implied parameters in closed form
sigma_m_sq=sigma_mu^2/(2*theta+kappa); %Asymptotic variance of posterior mean of drift rate
sigma_m = sqrt(sigma_m_sq);
sigma_D=sqrt(sigma_m_sq/kappa); %Standard deviation of dividend Wiener
K=sigma_m_sq/sigma_D; %Kalman gain


%% Supply parameters

gH=.08; %Aggregate population growth

%% Initial conditions

D0=1; %Initial dividend level
H0=1; %Current housing stock
N0=1; %Aggregate population size
mu_g = [mubar;mu0];
V0  = PV(mubar,rho,mubar,theta,sigma_mu,sigma_m,D0);
[gp0,Ii,gpf,If] = BGP_Rates(mu_g,eta); %BGP rates

%% Mortgage parameters

LTV=.87;
fore_disc=.645;
J=@(t) normcdf(t,aa,bb);
Jd=@(t) normpdf(t,aa,bb);
         
options = optimoptions('fsolve','SpecifyObjectiveGradient',true);
u_mult=3;
l_mult=.01;

LTV_dist=@(x) normpdf(x,LTV,.09); %Distribution of LTV at time of purchase
%LTV_dist=@(x) betapdf(x,8.2299,1.2297);
sol = gen_BGP_LTV_dist(gam,gp0,Ii,iota_L+iota_R,LTV_dist,l_mult,u_mult);

switch LTV_load_flag
    case 'on'
        load LTV_DATA
        max_LTV_DATA = max(LTV_DATA(:,1));
        F=griddedInterpolant(LTV_DATA(:,1),LTV_DATA(:,2),'pchip');
        func = @(x) F(x);
        [~,~,a,~] = cheb_create(6,0,max_LTV_DATA,func,0);
        LTV_cdf=chebfun(a,[0,max_LTV_DATA],'coeffs');
        LTV_BGP_cheb=diff(LTV_cdf);
        integral(LTV_BGP_cheb,0,max_LTV_DATA)
        fore_rate_BGP_cheb=iota_L*integral(@(x) LTV_BGP_cheb(x).*J(x),0,max_LTV_DATA);
        
        LTV_BGP = @(x) LTV_BGP_cheb(x).*(x<=max_LTV_DATA);
        fore_rate_BGP=iota_L*integral(@(x) LTV_BGP(x).*J(x),l_mult,u_mult);
        
    otherwise
        LTV_BGP = @(x) deval(sol,x);
        integral(LTV_BGP,l_mult,u_mult)
        fore_rate_BGP=iota_L*integral(@(x) LTV_BGP(x).*J(x),l_mult,u_mult);

end

%% Cost and Demand parameters - currently not used

%xm=((Ii)*H0/gN/N0)^(1/gamma_elas);
xm=((Ii+fore_rate_BGP)*H0/gH/H0)^(1/gamma_elas);
c=V0/H0^(1/eta);
land_share=.15;
A0=c*land_share;
B0=c*(1-land_share);
A_change=1.08;
Ag=log(1+A_change)/22;
B_change=0;
Bg=log(1+B_change)/22;
MAL=20;
alpha_Ibar=1/52/30;



%% Initial Loan Balance Distribution
[P0,I0,flag] = Solve_Eq_Price_BGP(V0,H0,N0,gH,c,eta,chi,xm,gamma_elas,Ii,0,fore_rate_BGP,1,10,options);

M_dist=@(M) H0*LTV_BGP(M/P0)/P0;
u_bound=u_mult*P0;
l_bound=l_mult*P0; %Support of mortgage balance distribution
u_bound_integral=2.5*P0;

def_func=@(P) iota_L*integral(@(M) M_dist(M).*J(M./P),l_bound,u_bound_integral);
def_deriv=@(P) iota_L*integral(@(M) -M_dist(M).*Jd(M./P).*M/(P.^2),l_bound,u_bound_integral);

[P0test,I0test,fore_test,fval,flag_init] = Solve_Eq_Price_Inv2(V0,H0,N0,gH,c,eta,chi,xm,gamma_elas,Ii,0,def_func,def_deriv,1,P0,options);
       
chebsize=50; %Number of collocation points
tol_adj=1e-12;

%% Save workspace variables 

save params

 




