% Load CBSA moments
clear;

addpath(genpath('/projectnb/gurengrp2/CRGM/SMM/shared_libraries'))
addpath(genpath('/Users/adam/Dropbox/Research/House Prices Growth Misperception (CRG)/Code'))

% Setup exogenous params
gam=0;
iota_tot = .045;
theta=0.005;
%mubar=0.010694293; 
mubar=.01496088; % Quartile 4
sigma_mu=.001;
LTV_load_flag = 'on';
save('exog_params','gam','iota_tot','theta','sigma_mu','LTV_load_flag');
T = 25;
zeta=0;
iota_R = 0;
nu = theta;
k=8;
aa=1;
cost_flag = 1;
nTrials_mc = 100;

% Load Moments
%data=readtable('../../Code/data_moments/cbsaquantile4_data-weighted.csv'); %for local
data= readtable('../data_moments/cbsaquantile4_data-weighted.csv'); %for server
for q=1:4

    eval(['boomstart = find(strcmp(data.quarterly,data.Q',num2str(q),'boomstart));'])
    eval(['peak = find(strcmp(data.quarterly,data.Q',num2str(q),'boomend));'])
    eval(['trough = find(strcmp(data.quarterly,data.Q',num2str(q),'bustend));'])
    boomstartcommon = find(strcmp(data.quarterly,'1998q1'));
    sampleend = find(strcmp(data.quarterly,'2019q4'));
    eval(['lhpi = log(data.Q',num2str(q),'HPIFM);']);
    eval(['H = data.Q',num2str(q),'unitsH;']);
    I = log(H./lagmatrix(H,4)) .* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since units are annual
    eval(['I = data.Q',num2str(q),'permitsI;']); %Use permits
    I = I./H; %Permits are Hdot
    I = I.* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since permits are annual
    I(I==0)=NaN;
    eval(['q',num2str(q),'hpi=data.Q',num2str(q),'HPIFM;']);

    boomsize_moment(q) = lhpi(peak) - lhpi(boomstart);
    bustsize_moment(q) = -1*[lhpi(trough) - lhpi(peak)];
    reboundsize_moment(q) = lhpi(trough+28) - lhpi(trough);
    bbrgrowth_moment(q) = (lhpi(sampleend) - lhpi(boomstartcommon))/((sampleend-(boomstartcommon))/4);
    boomconstruction_moment(q) = max(I(boomstartcommon:peak))/mean(I(boomstartcommon-20:boomstartcommon-4),'omitnan');
    bustconstruction_moment(q) = min(I(peak:trough+28))/max(I(boomstartcommon:peak));
    std_dP_bust_moment(q) = std(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    kurtosis_dP_bust_moment(q) = kurtosis(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    maxdprice_bust_moment(q) = min(lhpi(peak:trough) - lhpi(peak-4:trough-4));
    maxdprice_rebound_moment(q) = max(lhpi(trough:trough+28) - lhpi(trough-4:trough+24));
    eval(['boomlength_moment(q) = data.Q',num2str(q),'boomlength(1);'])
    eval(['bustlength_moment(q) = data.Q',num2str(q),'bustlength(1);'])
    eval(['maxforeclosure_moment(q) = data.Q',num2str(q),'max_foreclosure_rate(1)/100;']);
    eval(['yr9_negeq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt0(1);']);
    eval(['yr9_10eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt10(1);']);
    eval(['yr9_20eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt20(1);']);
    % No expectations moments by quartile.
    mubar(q) = (lhpi(88)-lhpi(9))/((88-9)/4);
end
eta_moment = 1./[data.Q1invelasticity(1),data.Q2invelasticity(1),data.Q3invelasticity(1),data.Q4invelasticity(1)];
expectation_2006_moment = .05;
expectation_2019_moment = .031;
bb_moment = [.09,.09,.09,.09];

% Load solver solution
%load('../../SMM/Diagnostic/optimal_params_xc.mat'); %for local
load('optimal_params_xc.mat'); %for server

% run things
Time = [0:1/52:T];
moments = zeros(17,4);
sim_momments = zeros(17,4);
delta = 0;

for q=4:-1:1
%MAIN RUN
    [P,I,Fore_rate,~,m_phi,~,~,flags,loan_dist_cheb,ubound_cheb,lbound_cheb,Hdot,~,~,D,R10,Rent_mp,Price_Rent,delta_sol,Pf]=...
                run_model_smm_rents(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc,delta);
    
    D_mat(:,q) = D;
    m_phi_mat(:,q) = m_phi;
    P_mat(:,q) = P;
    Pf_mat(:,:,q) = Pf;
    Rent_mat(:,q) = Rent_mp;
    delta_mat(:,q) = delta_sol;
    delta = delta_sol;
end

save('postgrid_rents.mat','D_mat','m_phi_mat','P_mat','Pf_mat','Rent_mat','delta_mat')
