function [P,R1,R10,Rent,Rent_mp,Price_Rent,I,W,m_phi,mu_true,mu_rational,flags,loan_dist_cheb,ubound_cheb,Hdot,V,H,D]...
            =run_model_smm_forecast(mu0,mubar,sigma_mu,theta,phi,nu,k,kappa,chi,iota_L,iota_R,rho,eta,aa,bb,gamma_elas,nTrials_mc,delta)
    % Runs model for SMM
    
     %Exogenously calibrated parameters
    load exog_params;
    
    save('passed_params','chi','gamma_elas','eta','gam','mu0','mubar','sigma_mu','theta',...
        'nu','k','kappa','phi','iota_L','iota_R','aa','bb','rho','LTV_load_flag','nTrials_mc','delta');

    %% Set up paramaters

    clear;
    addpath(genpath(pwd));
    run gen_parameters_passparams;
    
    %% Solve global equilibrium
    display('Solve Global')
    %nTrials=int16(100); 
    nTrials_mc=int16(nTrials_mc);%Number of trials for Monte-Carlo
    T=30; %Years
    ff=365; %Step-size
    tol=1e-3; %Stopping criteria
    max_its=50; %Max number of iterations
    w=3; %Level of the Smolyak grid
    approx_flag='linear'; %Use linear approximation ('linear') of polynomial interpolation ('global')
    [W,bP,bI,bW,bP1,bP10] = solve_global_forecast(T,ff,nTrials_mc,approx_flag,tol,max_its,w);
    clear T ff

    %% Determine initial loan balance distribution
    X_vec=[D0 mubar H0 Ii 1];
    W=X_vec*bW;
    c=(V0-W)/H0^(1/eta);
    
    [P0,I0,flag] = Solve_Eq_Price_BGP(V0,H0,N0,gH,c,eta,chi,xm,gamma_elas,Ii,W,fore_rate_BGP,1,10,options);

    M_dist=@(M) H0*LTV_BGP(M/P0)/P0;
    u_bound=u_mult*P0;
    l_bound=l_mult*P0; %Support of mortgage balance distribution
    u_bound_integral=2.5*P0;

    F = iota_L*integral(@(M) M_dist(M).*J(M./P0),l_bound,u_bound_integral);

    [P0test,I0test,flag_init] = Solve_Eq_Price_Impulse(V0,H0,N0,gH,c,eta,chi,xm,gamma_elas,Ii,W,F,1,P0,options);
    save params
    
    %% Solve impulse response function
    T=25; %Years
    ff=52; %Step-size
    [X,V,P,P1,P10,I,Ibar,a,W,F,mu_true,mu_rational,flags,L,U,PP,M_nodes] = simulate_impulse_forecast(T,ff,W,bW,bP1,bP10);

    %Collect results

    D=X(:,1);
    m_phi=X(:,2);
    H=X(:,3);
    H_pre = ones(52,1);
    H_pre(52) = H0-H0*Ii/52;
    for(t=51:-1:1)
        H_pre(t) = H_pre(t+1)-H0*Ii/52;
    end 
    Hdot = H - [H_pre;H(1:end-52)];

    P(end,:)=P(end-1,:);
    I(end,:)=I(end-1,:);
    H(end,:)=H(end-1,:);
    V=mean(V,2);
    P=mean(P,2);
    I=mean(I,2);
    
    X_eq=X;
    P_eq=P;
    I_eq=I;
    F_eq=F;
    a_eq=a;
    Ibar_eq=Ibar;

    %% Solve for price forecasts and rents
    Tf=30; %Simulation length for price beliefs
    [R1,R10,P1,P10,Pf] = priceBeliefsTest(X_eq,I_eq,Ibar_eq,a_eq,F_eq,P_eq,L,U,PP,M_nodes,ff,bW,...
                           mubar,gamma_elas,alpha_Ibar,sigma_D,iota_L,iota_R,u_bound_integral,...
                           theta,sigma_mu,sigma_m,N0,gH,c,eta,chi,xm,l_bound,u_bound,rho,chebsize,J,LTV_dist,gam,Tf);
    
    IR=rho;
%     RentPriceRatio=.1;
%     delta=(RentPriceRatio-(1-LTV)*rho-LTV*IR+gp0)*P0/D0;
    Delta=1/ff;
    lambda=1/6;
    Rent=((1-LTV)*rho+LTV*IR-R1).*P+delta*D;
    
    TT=0:1:Tf*ff;
    PV_Div=PV(m_phi,rho+lambda,mubar,theta,sigma_mu,sigma_m,D);

%     delta=(RentPriceRatio-(1-LTV)*rho-LTV*IR+lambda*gp0/(rho+lambda-gp0))*P0/D0*(rho+lambda-gp0)/(rho+lambda);
%     Rent_0=(rho-lambda*gp0/(rho+lambda-gp0))*P0+(rho+lambda)/(rho+lambda-gp0)*delta*D0;
%     Rent_T=(rho-lambda*gpf/(rho+lambda-gpf))*P(end)+(rho+lambda)/(rho+lambda-gpf)*delta*D(end);
% 
%     PriceGrowthExp=lambda*((rho+lambda)*Delta*sum(((1-lambda*Delta)/(1+rho*Delta)).^TT.*Pf,2)-P)./P;
%     Rent_reset=((1-LTV)*rho+LTV*IR-PriceGrowthExp).*P+(rho+lambda)./(rho+lambda-m_phi).*delta.*D;
    
    %gR=zeros(size(m_phi));
    gR=m_phi;
    %delta=(RentPriceRatio-(1-LTV)*rho-LTV*IR+lambda*gp0/(rho+lambda-gp0))*P0/PV_Div(1)/(rho+lambda);
    %delta=(RentPriceRatio-(1-LTV)*rho-LTV*IR+gp0)*P0/D0;
    Rent_0=(rho-lambda*gp0/(rho+lambda-gp0))*P0+(rho+lambda)*delta*PV_Div(1);
    Rent_T=(rho-lambda*gpf/(rho+lambda-gpf))*P(end)+(rho+lambda)*delta*PV_Div(end);
    
    PriceGrowthExp=lambda*((rho+lambda)*Delta*sum(((1-lambda*Delta)/(1+rho*Delta)).^TT.*Pf,2)-P)./P;
    Rent_reset=(rho+lambda-gR)./(rho+lambda).*((1-LTV)*rho+LTV*IR-PriceGrowthExp).*P+(rho+lambda-gR).*delta.*PV_Div;

    Rent_mp=zeros(size(Rent_reset));
    Rent_mp(1)=Rent_reset(1);
    Masses=zeros(size(Rent_reset));
    Masses(1)=1;
    for t=2:size(Rent_mp,1)
       Masses=(1-lambda*Delta)*Masses;
       Masses(t)=lambda*Delta;
       T_vec=(t-1:-1:t-1-T*ff)';
       T_vec=max(T_vec,0);
       Rent_vec=Rent_reset.*(1+gR*Delta).^T_vec;
       Rent_mp(t)=sum(Masses.*Rent_vec);
       %Rent_mp(t)=(1-lambda*Delta)*Rent_mp(t-1)+lambda*Delta*Rent_reset(t); 
    end
    Price_Rent=P./Rent_mp;                   
%     R1=P1./P-1;
%     R10=(P10./P).^(1/10)-1;
%% 

    
    % Loan balance dist every year
    annual = [1:52:T*52+1];
    loan_dist_cheb = a(:,annual);
    ubound_cheb = u_bound;
end