% Load CBSA moments
clear;

addpath(genpath('/projectnb/gurengrp2/CRGM/SMM/shared_libraries'))
addpath(genpath('/Users/adam/Dropbox/Research/House Prices Growth Misperception (CRG)/Code'))

% Setup exogenous params
gam=0;
iota_tot = .045;
theta=0.005;
%mubar=0.010694293; 
mubar=.01496088; % Quartile 4
sigma_mu=.001;
LTV_load_flag = 'on';
save('exog_params','gam','iota_tot','theta','sigma_mu','LTV_load_flag');
T = 25;
zeta=0;
iota_R = 0;
nu = theta;
k=8;
aa=1;
cost_flag = 1;
nTrials_mc = 100;

% Load Moments
data= readtable('../data_moments/cbsaquantile4_data-weighted.csv');
for q=1:4

    eval(['boomstart = find(strcmp(data.quarterly,data.Q',num2str(q),'boomstart));'])
    eval(['peak = find(strcmp(data.quarterly,data.Q',num2str(q),'boomend));'])
    eval(['trough = find(strcmp(data.quarterly,data.Q',num2str(q),'bustend));'])
    boomstartcommon = find(strcmp(data.quarterly,'1998q1'));
    sampleend = find(strcmp(data.quarterly,'2019q4'));
    eval(['lhpi = log(data.Q',num2str(q),'HPIFM);']);
    eval(['H = data.Q',num2str(q),'unitsH;']);
    I = log(H./lagmatrix(H,4)) .* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since units are annual
    eval(['I = data.Q',num2str(q),'permitsI;']); %Use permits
    I = I./H; %Permits are Hdot
    I = I.* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since permits are annual
    I(I==0)=NaN;
    eval(['q',num2str(q),'hpi=data.Q',num2str(q),'HPIFM;']);

    boomsize_moment(q) = lhpi(peak) - lhpi(boomstart);
    bustsize_moment(q) = -1*[lhpi(trough) - lhpi(peak)];
    reboundsize_moment(q) = lhpi(trough+28) - lhpi(trough);
    bbrgrowth_moment(q) = (lhpi(sampleend) - lhpi(boomstartcommon))/((sampleend-(boomstartcommon))/4);
    boomconstruction_moment(q) = max(I(boomstartcommon:peak))/mean(I(boomstartcommon-20:boomstartcommon-4),'omitnan');
    bustconstruction_moment(q) = min(I(peak:trough+28))/max(I(boomstartcommon:peak));
    std_dP_bust_moment(q) = std(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    kurtosis_dP_bust_moment(q) = kurtosis(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    maxdprice_bust_moment(q) = min(lhpi(peak:trough) - lhpi(peak-4:trough-4));
    maxdprice_rebound_moment(q) = max(lhpi(trough:trough+28) - lhpi(trough-4:trough+24));
    eval(['boomlength_moment(q) = data.Q',num2str(q),'boomlength(1);'])
    eval(['bustlength_moment(q) = data.Q',num2str(q),'bustlength(1);'])
    eval(['maxforeclosure_moment(q) = data.Q',num2str(q),'max_foreclosure_rate(1)/100;']);
    eval(['yr9_negeq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt0(1);']);
    eval(['yr9_10eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt10(1);']);
    eval(['yr9_20eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt20(1);']);
    % No expectations moments by quartile.
    mubar(q) = (lhpi(88)-lhpi(9))/((88-9)/4);
end
eta_moment = 1./[data.Q1invelasticity(1),data.Q2invelasticity(1),data.Q3invelasticity(1),data.Q4invelasticity(1)];
expectation_2006_moment = .05;
expectation_2019_moment = .031;
bb_moment = [.09,.09,.09,.09];

% Load optimal params
load('optimal_params_xc.mat');

% run things
Time = [0:1/52:T];
moments = zeros(17,4);
sim_momments = zeros(17,4);

for q=1:4
%MAIN RUN
    [P,I,Fore_rate,~,m_phi,~,~,flags,loan_dist_cheb,ubound_cheb,lbound_cheb,Hdot,V,H,D,R10]=...
                run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
  
    Fore_rate(end) = Fore_rate(end-1);
    mu_true = mu0(q)*exp(-theta*Time) + mubar(q)*(1-exp(-theta*Time));
    
    P_mat(:,q) = P;
    I_mat(:,q) = I;
    Fore_rate_mat(:,q) = Fore_rate;
    m_phi_mat(:,q) = m_phi;
    mu_true_mat(:,q) = mu_true;

    %{
    subplot(2,2,1), plot(Time,P./P(1),'LineWidth',1.5);
    title('Price Index')
    subplot(2,2,2), plot(Time,I./I(1),'LineWidth',1.5);
    title('Construction')
    subplot(2,2,3), plot(Time,Fore_rate,'LineWidth',1.5);
    title('Foreclosure Rate')
    subplot(2,2,4), plot(Time,m_phi,'LineWidth',1.5);
    hold on;
    subplot(2,2,4), plot(Time,mu_true,'LineStyle','--','Color','black');
    hold off;
    title('Beliefs')
    %}

    [boomsize(q),bustsize(q),reboundsize(q),bbrgrowth(q),boomlength(q),bustlength(q),maxforeclosure(q),std_dP_bust(q),kurtosis_dP_bust(q),maxdprice_bust(q),maxdprice_rebound(q)] = ...
        calc_moments(Time,P,Fore_rate)
    M_pdf=chebfun(loan_dist_cheb(:,10),[lbound_cheb,ubound_cheb],'coeffs'); %beginning of year 9 is the 10th one (start at zero)
    yr9_negeq(q) = sum(M_pdf([P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
    yr9_10eq(q) = sum(M_pdf([.9*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
    yr9_20eq(q) = sum(M_pdf([.8*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));

    % Expectations
    expectation_2006(q) = R10(1);
    expectation_2019(q) = R10(2);
    m_phi_2006(q) = m_phi(52*9+1);
    m_phi_2019(q) = m_phi(52*22+1);
    


    % Calculate SSR
    if q==4 
        moments(:,q) = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
                boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
                kurtosis_dP_bust_moment(q);maxdprice_bust_moment(q);...
                maxdprice_rebound_moment(q);bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);...
                expectation_2006_moment;expectation_2019_moment];
        weights=[0,0,0,bbrweight(q),0,1,1,0,0,1,0,1,1/3,1/3,1/3,1,1]; 
        
    else
        moments(:,q) = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
            boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
                kurtosis_dP_bust_moment(q);maxdprice_bust_moment(q);...
                maxdprice_rebound_moment(q);bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);1;1];
        weights=[0,0,0,bbrweight(q),0,1,1,0,0,1,0,1,1/3,1/3,1/3,0,0];
    end
    weights = weights/sum(weights);
    sim_moments(:,q) = [boomsize(q);bustsize(q);reboundsize(q);bbrgrowth(q);boomlength(q);bustlength(q);maxforeclosure(q);...
            std_dP_bust(q);kurtosis_dP_bust(q);maxdprice_bust(q);maxdprice_rebound(q);bb(q);...
            yr9_negeq(q);yr9_10eq(q);yr9_20eq(q);expectation_2006(q);expectation_2019(q)];  
    ssr_sim(q) = weights*(((sim_moments(:,q)-moments(:,q))./moments(:,q)).^2)
end

format short
ssr
ssr_sim

sim_moments
moments
deviation = (sim_moments - moments)./moments

eta
mu0
mubar
mu0 - mubar
kappa
nu
k
phi
chi
iota_L
rho
aa
bb
gamma_elas
%exitflag_mat

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plot of best fit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
year = [1975:.25:2020.75];
yrs = find(year >= 1998 & year <= 2020);

subplot(2,2,1), plot(Time,P_mat(:,1)./P_mat(1,1),'LineWidth',1.5);
hold on;
subplot(2,2,1), plot([0:.25:22],q1hpi(yrs)./q1hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--')
%subplot(2,2,1), text(7,2.25,['$\mu_0$ = ',num2str(mu0(1)),'\%, $\chi$ = ',num2str(chi(1)),'\%, $\iota$ =', num2str(iota_L(1))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
h = legend('Model','Data','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');
title('Quartile 1 (Smallest)','Interpreter','latex','FontSize',10);
ylim([.9,3]);
ylabel('Price Index','Interpreter','latex');

subplot(2,2,2), plot(Time,P_mat(:,2)./P_mat(1,2),'LineWidth',1.5);
hold on;
subplot(2,2,2), plot([0:.25:22],q2hpi(yrs)./q2hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--');
%subplot(2,2,2), text(7,2.25,['$\mu_0$ = ',num2str(mu0(2)),'\%, $\chi$ = ',num2str(chi(2)),'\%, $\iota$ =', num2str(iota_L(2))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Quartile 2','Interpreter','latex','FontSize',10);
ylim([.9,3]);
ylabel('Price Index','Interpreter','latex');

subplot(2,2,3), plot(Time,P_mat(:,3)./P_mat(1,3),'LineWidth',1.5);
hold on;
subplot(2,2,3), plot([0:.25:22],q3hpi(yrs)./q3hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--');
%subplot(2,2,3), text(7,2.25,['$\mu_0$ = ',num2str(mu0(3)),'\%, $\chi$ = ',num2str(chi(3)),'\%, $\iota$ =', num2str(iota_L(3))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Quartile 3','Interpreter','latex','FontSize',10);
ylim([.9,3]);
xlabel('Years','Interpreter','latex');
ylabel('Price Index','Interpreter','latex');

subplot(2,2,4), plot(Time,P_mat(:,4)./P_mat(1,4),'LineWidth',1.5);
hold on;
subplot(2,2,4), plot([0:.25:22],q4hpi(yrs)./q4hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--');
%subplot(2,2,4), text(7,1.1,['$\mu_0$ = ',num2str(mu0(4)),'\%, $\chi$ = ',num2str(chi(4)),'\%, $\iota$ =', num2str(iota_L(4))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Quartile 4','Interpreter','latex','FontSize',10);
ylim([.9,3]);
xlabel('Years','Interpreter','latex');
ylabel('Price Index','Interpreter','latex');

set(gcf, 'Units', 'Inches', 'Position', [0, 0, 5.039, 3.10], 'PaperUnits', 'Inches', 'PaperSize', [5.039, 3.10])
saveas(gcf,'output/slides_crosscities.pdf')
set(gcf, 'Units', 'Inches', 'Position', [0, 0, 5.75, 3.5], 'PaperUnits', 'Inches', 'PaperSize', [5.75, 3.5])
saveas(gcf,'output/paper_crosscities.pdf')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Table of data and moments
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[nmoments,nq] = size(sim_moments);
table_moments = zeros(nmoments,2*nq);
colnames = repmat("",1,2*nq);
for q = (1:nq) %Input Data Achieved
    table_moments(:,2*q-1) = moments(:,q);
    table_moments(:,2*q) = sim_moments(:,q);    
    colnames(2*q-1:2*q) = ["Data","Achieved"];
end
table_moments(11,:)= []; % Remove rebound speed
table_moments(9,:)= []; % Remove kurtosis dP bust
table_moments(8,:)= []; % Remove std dP bust
table_moments(5,:)=[]; %Remove boom length
table_moments(2,:) = table_moments(2,:) * -1; % Make bust negative
table

%{
REORDER
1->11. boomsize
2->12. bustsize
3->13. reboundsize
4->1. bbrgrowth
5->4. bust length
6->5. max foreclosures
7->9. maxpdrice bust
8->10. bb
9->8. yr9 negeq 
10->7. yr9 10eq
11->6. yr9 20eq
12->2. expectation 2006
13->3. expectation 2019
%}
table_moments = table_moments([4 12 13 5 6 11 10 9 7 8 1 2 3],:); %Reorder rows

%{
NEW ORDER
1. bbgrowth
2. expectation 2006
3. expectation 2019
4. bust length
5. max foreclosrues
6. yr9 20eq
7. yr9 10eq
8. yr9 negeq
9. maxdprice bust
10. bb
11. boomsize
12. bustsize
13. reboundsize
%}
source = ["","Freddie Mac &","Shiller-Thompson &", "Shiller-Thompson &","","Freddie Mac &","",...
    "CoreLogic &","","Beraja et al. &","Beraja et al. &","Beraja et al. &",...
    "Freddie Mac &","","DataQuick &","","Freddie Mac &", "Freddie Mac &", "Freddie Mac &"]';
rownames = ["\hline\multicolumn{8}{l}{\emph{Long run house price growth and expectations (annualized log points)}}\\",...
    "1. 1997-2019 HPI &","2. Expectations 2006 &","3. Expectations 2019 &",...
    "\hline\multicolumn{8}{l}{\emph{Phase length (years)}}\\","4. Bust &",...
    "\hline\multicolumn{8}{l}{\emph{Role of foreclosures}}\\",...
    "5. Max Foreclosure Rate &","\\",...
    "6. \% Jan-07 Equity$<20$\% &","7. \% Jan-07 Equity$<10$\% &","8. \% Jan-07 Equity$<0$\% &",...
    "9. Bust Speed (log points) &","\hline\multicolumn{8}{l}{\emph{Standard deviation of house price shocks (log points)}\\",...
    "10. Repeat Sales Resid SD & ","\hline\multicolumn{8}{l}{\emph{Non-targeted short run house price growth (unannualized log points)}}\\",...
    "\hspace{2mm}  Boom & ", "\hspace{2mm}  Bust & ", "\hspace{2mm}  Rebound & "]';
colnames = ["Moment &","Source &",colnames(1:2*q-1)+" &",colnames(2*q)+" \\"];

table_moments([1:3 5:13],:) = table_moments([1:3 5:13],:)*100; %Convert to percent
table_moments([1:3 6:13],:) = round(table_moments([1:3 6:13],:),1); %Show one decimal
table_moments(4:5,:) = round(table_moments(4:5,:),2); %Show two decimals
table_moments = table_moments+"";
table_moments(2:3,1:6) = repmat("",2,6); % zero out expectations in q1-q3.
table_moments(:,1:2*nq-1) = table_moments(:,1:2*nq-1) + "&"; %For tex
table_moments(:,2*nq) = table_moments(:,2*nq) + "\\"; %For tex
table_moments = [repmat("",1,2*nq);table_moments(1:3,:);repmat("",1,2*nq);...
    table_moments(4,:);repmat("",1,2*nq);table_moments(5,:);repmat("",1,2*nq);...
    table_moments(6:9,:);repmat("",1,2*nq);table_moments(10,:);repmat("",1,2*nq);table_moments(11:13,:)]; %Add empty rows for label
%{
table_moments = [table_moments;repmat("",1,2*nq)]; %Initiate row for root mean squared residual
for q = (1:nq) %Input root mean squared residual 
    table_moments(size(table_moments,1),2*q-1) = "\multicolumn{2}{c}{" + round(100*sqrt(ssr(q)),1) + "}";
    if q<nq
       table_moments(size(table_moments,1),2*q-1) = table_moments(size(table_moments,1),2*q-1) + " &";
    else 
        table_moments(size(table_moments,1),2*q-1) = table_moments(size(table_moments,1),2*q-1) + "\\";
    end
end
%}
table_moments = [rownames,source,table_moments];
table_moments = [colnames;table_moments];
writematrix(table_moments,'output/moments.txt','Delimiter','space');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Rent Figure With Dividends
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Compute dividend path
for q=1:4
    D(:,q) = dividends(Time,mu0(q),mubar(q),theta,sigma_mu,1);
end
D = D./D(1,:);

year = [1975:.25:2020.75];
yrs = find(year >= 1998 & year <= 2020);
% Rent data is only biannual so need to pick out those
selector = isnan(data.Q1cpirent(yrs))==0;
yrs2 = yrs(selector);

P_mat2 = P_mat;
load('postgrid_rents.mat')

subplot(2,2,1), plot(Time,Rent_mat(:,1)./Rent_mat(1,1),'LineWidth',1.5);
hold on;
subplot(2,2,1), plot([0:.5:22],data.Q1cpirent(yrs2)./data.Q1cpirent(min(yrs2)),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,1), plot(Time,D(:,1),'LineWidth',1.5,'LineStyle',':');
axis([0 22 -inf inf])
xticks([0 5 10 15 20 25]);
yticks([1 1.5 2 2.5]);
hold off;
h = legend('Model rent','Data rent','Model dividend','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');
title('(a) Quartile 1 (Smallest)','Interpreter','latex','FontSize',10);
ylim([.7,3]);
ylabel('Rent Index','Interpreter','latex');
model_q1 = Rent_mat(Time==22,1)./Rent_mat(1,1)
data_q1 = data.Q1cpirent(yrs2(end))./data.Q1cpirent(min(yrs2))

subplot(2,2,2), plot(Time,Rent_mat(:,2)./Rent_mat(1,2),'LineWidth',1.5);
hold on;
subplot(2,2,2), plot([0:.5:22],data.Q2cpirent(yrs2)./data.Q2cpirent(min(yrs2)),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,2), plot(Time,D(:,2),'LineWidth',1.5,'LineStyle',':');
axis([0 22 -inf inf])
xticks([0 5 10 15 20 25]);
yticks([1 1.5 2 2.5]);
hold off;
title('(b) Quartile 2','Interpreter','latex','FontSize',10);
ylim([.7,3]);
ylabel('Rent Index','Interpreter','latex');
model_q2 = Rent_mat(Time==22,2)./Rent_mat(1,2)
data_q2 = data.Q2cpirent(yrs2(end))./data.Q2cpirent(min(yrs2))

subplot(2,2,3), plot(Time,Rent_mat(:,3)./Rent_mat(1,3),'LineWidth',1.5);
hold on;
subplot(2,2,3), plot([0:.5:22],data.Q3cpirent(yrs2)./data.Q3cpirent(min(yrs2)),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,3), plot(Time,D(:,3),'LineWidth',1.5,'LineStyle',':');
axis([0 22 -inf inf])
xticks([0 5 10 15 20 25]);
yticks([1 1.5 2 2.5]);
hold off;
title('(c) Quartile 3','Interpreter','latex','FontSize',10);
ylim([.7,3]);
xlabel('Years','Interpreter','latex');
ylabel('Rent Index','Interpreter','latex');
model_q3 = Rent_mat(Time==22,3)./Rent_mat(1,3)
data_q3 = data.Q3cpirent(yrs2(end))./data.Q3cpirent(min(yrs2))

subplot(2,2,4), plot(Time,Rent_mat(:,4)./Rent_mat(1,4),'LineWidth',1.5);
hold on;
subplot(2,2,4), plot([0:.5:22],data.Q4cpirent(yrs2)./data.Q4cpirent(min(yrs2)),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,4), plot(Time,D(:,4),'LineWidth',1.5,'LineStyle',':');
axis([0 22 -inf inf])
yticks([1 1.5 2 2.5]);
hold off;
title('(d) Quartile 4 (Largest)','Interpreter','latex','FontSize',10);
ylim([.7,3]);
xlabel('Years','Interpreter','latex');
ylabel('Rent Index','Interpreter','latex');
model_q4 = Rent_mat(Time==22,4)./Rent_mat(1,4)
data_q4 = data.Q4cpirent(yrs2(end))./data.Q4cpirent(min(yrs2))

set(gcf, 'Units', 'Inches', 'Position', [0, 0, 5.75, 3.5], 'PaperUnits', 'Inches', 'PaperSize', [5.75, 3.5])
saveas(gcf,'output/paper_rents.pdf')
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Figure where turn off foreclosures and learning
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
q = 4;
% Run No Foreclosure
[P_nf,I_nf,Fore_rate_nf,~,m_phi_nf]=...
       run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),0,iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
Fore_rate_nf(end) = Fore_rate_nf(end-1);
[boomsize_nf,bustsize_nf,reboundsize_nf] = calc_moments(Time,P_nf,Fore_rate_nf)

% Run no learning
[P_nl,I_nl,Fore_rate_nl,~,m_phi_nl]=...
       run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,0,nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
Fore_rate_nl(end) = Fore_rate_nl(end-1);

% Full Info Rational Expectations
[P_fire,I_fire,Fore_rate_fire,~,m_phi_fire]=...
       run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,0,nu,k,1000,chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
Fore_rate_fire(end) = Fore_rate_fire(end-1);

%{
subplot(2,2,1), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(2,2,1), plot(Time,P_nf./P_nf(1),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,1), plot(Time,P_nl./P_nl(1),'LineWidth',1.5,'LineStyle','-.');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(a) Price Index','Interpreter','latex','FontSize',10)
subplot(2,2,2), plot(Time,I./I(1),'LineWidth',1.5);
hold on;
subplot(2,2,2), plot(Time,I_nf./I_nf(1),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,2), plot(Time,I_nl./I_nl(1),'LineWidth',1.5,'LineStyle','-.');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
h = legend('Baseline','$\iota=0$','$\theta=0$','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');
title('(b) Construction','Interpreter','latex','FontSize',10)
subplot(2,2,3), plot(Time,Fore_rate,'LineWidth',1.5);
hold on;
subplot(2,2,3), plot(Time,Fore_rate_nf,'LineWidth',1.5,'LineStyle','--');
subplot(2,2,3), plot(Time,Fore_rate_nl,'LineWidth',1.5,'LineStyle','-.');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25])
yticks([0 0.005 0.01 0.015])
ax = gca
ax.YAxis.Exponent = 0;
hold off;
title('(c) Foreclosure Rate','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
subplot(2,2,4), plot(Time,m_phi,'LineWidth',1.5)
hold on;
subplot(2,2,4), plot(Time,m_phi_nf,'LineWidth',1.5,'LineStyle','--');
subplot(2,2,4), plot(Time,m_phi_nl,'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,4), plot(Time,mu_true,'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(d) Beliefs','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
set(gcf, 'Units', 'Inches', 'Position', [0, 0, 5.039, 3.10], 'PaperUnits', 'Inches', 'PaperSize', [5.039, 3.10])
saveas(gcf,'output/slides_noforeclosurediagnostic.pdf')
set(gcf, 'Units', 'Inches', 'Position', [0, 0, 5.75, 3.5], 'PaperUnits', 'Inches', 'PaperSize', [5.75, 3.5])
saveas(gcf,'output/paper_noforeclosurediagnostic.pdf')
%}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Changing rho and mubar figure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%{
% LOW RHO     
[P_lr]=run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q)-.0025,eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
% HIGH RHO
[P_hr]=run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q)+.035,eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
% High mubar
[P_hm]=run_model_smm_beliefs(mu0(q)+.0025,mubar(q)+.0025,sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
% Low mu
[P_lm]=run_model_smm_beliefs(mu0(q)-.0025,mubar(q)-.0025,sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);

% Make Figure
subplot(1,2,1), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(1,2,1), plot(Time,P_lr./P_lr(1),'LineWidth',1.5,'LineStyle','--');
subplot(1,2,1), plot(Time,P_hr./P_hr(1),'LineWidth',1.5,'LineStyle','-.');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(a) Changing Initial $\rho$','Interpreter','latex','FontSize',10)
h = legend('Baseline $\rho = 3.96\%$','$\rho = 3.71\%$','$\rho = 4.21\%$','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');

subplot(1,2,2), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(1,2,2), plot(Time,P_hm./P_hm(1),'LineWidth',1.5,'LineStyle','--');
subplot(1,2,2), plot(Time,P_lm./P_lm(1),'LineWidth',1.5,'LineStyle','-.');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(a) Changing $\bar{\mu}$','Interpreter','latex','FontSize',10)
h = legend('Baseline $\bar{\mu} = 1.51\%$','$\bar{\mu} = 1.76\%$','$\bar{\mu} = 1.26\%$','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');

set(gcf, 'Units', 'Inches', 'Position', [0, 0, 8, 3], 'PaperUnits', 'Inches', 'PaperSize', [8,3])
saveas(gcf,'output/paper_rho_mubar.pdf')
set(gcf, 'Units', 'Inches', 'Position', [0, 0, 5.039, 2.25], 'PaperUnits', 'Inches', 'PaperSize', [5.039, 2.25])
saveas(gcf,'output/slides_rho_mubar.pdf')
%}

[P_hr]=run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q)+.04,eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Combined figure where turn off foreclosures and learning with rho
% "Role of Structural Elements"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Fore_rate = Fore_rate * 100;
Fore_rate_nf = Fore_rate_nf * 100;
Fore_rate_nl = Fore_rate_nl * 100;
Fore_rate_fire = Fore_rate_fire * 100;

m_phi = m_phi * 100;
m_phi_nf = m_phi_nf * 100;
m_phi_nl = m_phi_nl * 100;
mu_true = mu_true * 100;

subplot(2,2,1), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(2,2,1), plot(Time,P_nf./P_nf(1),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,1), plot(Time,P_nl./P_nl(1),'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,1), plot(Time,P_fire./P_fire(1),'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(a) Price Index','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Price Index','Interpreter','latex')
h = legend('Baseline','$\iota=0$','$\theta=0$','FIRE','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');

subplot(2,2,2), plot(Time,Fore_rate,'LineWidth',1.5);
hold on;
subplot(2,2,2), plot(Time,Fore_rate_nf,'LineWidth',1.5,'LineStyle','--');
subplot(2,2,2), plot(Time,Fore_rate_nl,'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,2), plot(Time,Fore_rate_fire,'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25])
yticks([0 .5 1 1.5 2])
ax = gca
ax.YAxis.Exponent = 0;
hold off;
title('(b) Foreclosure Rate','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Pct of Stock, Annual','Interpreter','latex')
h = legend('Baseline','$\iota=0$','$\theta=0$','FIRE','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');

subplot(2,2,3), plot(Time,m_phi,'LineWidth',1.5)
hold on;
subplot(2,2,3), plot(Time,m_phi_nf,'LineWidth',1.5,'LineStyle','--');
subplot(2,2,3), plot(Time,m_phi_nl,'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,3), plot(Time,mu_true,'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(c) Beliefs About Drift Rate $\mu$','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Percent','Interpreter','latex')
h = legend('Baseline','$\iota=0$','$\theta=0$','FIRE','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');


subplot(2,2,4), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(2,2,4), plot(Time,P_hr./P_hr(1),'LineWidth',1.5,'LineStyle','--');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(d) Price Index, Varying $\rho$','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Price Index','Interpreter','latex')
h = legend('Baseline $\rho = 7.75\%$','$\rho = 11.75\%$','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');

set(gcf, 'Units', 'Inches', 'Position', [0, 0, 8.625, 5.25], 'PaperUnits', 'Inches', 'PaperSize', [8.625, 5.25])
saveas(gcf,'output/paper_structuralcombined.pdf')


subplot(2,2,1), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(2,2,1), plot(Time,P_nf./P_nf(1),'LineWidth',1.5,'LineStyle','--');
subplot(2,2,1), plot(Time,P_nl./P_nl(1),'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,1), plot(Time,P_fire./P_fire(1),'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(a) Price Index','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Price Index','Interpreter','latex')

subplot(2,2,2), plot(Time,Fore_rate,'LineWidth',1.5);
hold on;
subplot(2,2,2), plot(Time,Fore_rate_nf,'LineWidth',1.5,'LineStyle','--');
subplot(2,2,2), plot(Time,Fore_rate_nl,'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,2), plot(Time,Fore_rate_fire,'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25])
yticks([0 .5 1 1.5 2])
ax = gca
ax.YAxis.Exponent = 0;
hold off;
title('(b) Foreclosure Rate','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Pct of Stock, Annual','Interpreter','latex')
h = legend('Baseline','$\iota=0$','$\theta=0$','FIRE','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');

subplot(2,2,3), plot(Time,m_phi,'LineWidth',1.5)
hold on;
subplot(2,2,3), plot(Time,m_phi_nf,'LineWidth',1.5,'LineStyle','--');
subplot(2,2,3), plot(Time,m_phi_nl,'LineWidth',1.5,'LineStyle','-.');
subplot(2,2,3), plot(Time,mu_true,'LineStyle','--','Color','black');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(c) Beliefs About Drift Rate $\mu$','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Percent','Interpreter','latex')
 
subplot(2,2,4), plot(Time,P./P(1),'LineWidth',1.5);
hold on;
subplot(2,2,4), plot(Time,P_hr./P_hr(1),'LineWidth',1.5,'LineStyle','--');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('(d) Price Index, Varying $\rho$','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
ylabel('Price Index','Interpreter','latex')
h = legend('Baseline $\rho = 7.75\%$','$\rho = 11.75\%$','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');


set(gcf, 'Units', 'Inches', 'Position', [0, 0, 6.75, 4.05], 'PaperUnits', 'Inches', 'PaperSize', [6.75, 4.05])
saveas(gcf,'output/slides_structuralcombined.pdf')