% Load CBSA moments
clear;

addpath(genpath('/projectnb/gurengrp2/CRGM/SMM/shared_libraries'))
addpath(genpath('/Users/adam/Dropbox/Research/House Prices Growth Misperception (CRG)/Code'))
addpath(genpath('../'))

% Setup exogenous params
gam=0;
iota_tot = .045;
theta=0.005;
%mubar=0.010694293; 
mubar=.01496088; % Quartile 4
sigma_mu=.001;
LTV_load_flag = 'on';
save('exog_params','gam','iota_tot','theta','sigma_mu','LTV_load_flag');
T = 25;
zeta=0;
iota_R = 0;
nu = theta;
k=8;
aa=1;
cost_flag = 1;

% Load Moments
data= readtable('../data_moments/cbsaquantile4_data-weighted.csv');
for q=1:4

    eval(['boomstart = find(strcmp(data.quarterly,data.Q',num2str(q),'boomstart));'])
    eval(['peak = find(strcmp(data.quarterly,data.Q',num2str(q),'boomend));'])
    eval(['trough = find(strcmp(data.quarterly,data.Q',num2str(q),'bustend));'])
    boomstartcommon = find(strcmp(data.quarterly,'1998q1'));
    sampleend = find(strcmp(data.quarterly,'2019q4'));
    eval(['lhpi = log(data.Q',num2str(q),'HPIFM);']);
    eval(['H = data.Q',num2str(q),'unitsH;']);
    I = log(H./lagmatrix(H,4)) .* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since units are annual
    eval(['I = data.Q',num2str(q),'permitsI;']); %Use permits
    I = I./H; %Permits are Hdot
    I = I.* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since permits are annual
    I(I==0)=NaN;
    eval(['q',num2str(q),'hpi=data.Q',num2str(q),'HPIFM;']);

    boomsize_moment(q) = lhpi(peak) - lhpi(boomstart);
    bustsize_moment(q) = -1*[lhpi(trough) - lhpi(peak)];
    reboundsize_moment(q) = lhpi(trough+28) - lhpi(trough);
    bbrgrowth_moment(q) = (lhpi(sampleend) - lhpi(boomstartcommon))/((sampleend-(boomstartcommon))/4);
    boomconstruction_moment(q) = max(I(boomstartcommon:peak))/mean(I(boomstartcommon-20:boomstartcommon-4),'omitnan');
    bustconstruction_moment(q) = min(I(peak:trough+28))/max(I(boomstartcommon:peak));
    std_dP_bust_moment(q) = std(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    kurtosis_dP_bust_moment(q) = kurtosis(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    maxdprice_bust_moment(q) = min(lhpi(peak:trough) - lhpi(peak-4:trough-4));
    maxdprice_rebound_moment(q) = max(lhpi(trough:trough+28) - lhpi(trough-4:trough+24));
    eval(['boomlength_moment(q) = data.Q',num2str(q),'boomlength(1);'])
    eval(['bustlength_moment(q) = data.Q',num2str(q),'bustlength(1);'])
    eval(['maxforeclosure_moment(q) = data.Q',num2str(q),'max_foreclosure_rate(1)/100;']);
    eval(['yr9_negeq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt0(1);']);
    eval(['yr9_10eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt10(1);']);
    eval(['yr9_20eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt20(1);']);
    % No expectations moments by quartile.
    mubar(q) = (lhpi(88)-lhpi(9))/((88-9)/4);
end
eta_moment = 1./[data.Q1invelasticity(1),data.Q2invelasticity(1),data.Q3invelasticity(1),data.Q4invelasticity(1)];
expectation_2006_moment = .05;
expectation_2019_moment = .031;
bb_moment = [.09,.09,.09,.09];

% Load solver solution
load('optimal_params_xc.mat');

q = 4;

%{
mu0=mu0(q);
mubar=mubar(q);
phi=phi(q);
kappa=kappa(q);
chi=chi(q);
iota_L=iota_L(q);
rho=rho(q);
eta=eta(q);
bb=bb(q);
gamma_elas=gamma_elas(q);
fore_disc=.645;
%}
%MAIN RUN
q = 4
nTrials_mc=100
[P_main,I_main,Fore_rate_main,W_main,m_phi_main,mu_true_main,mu_rational_main,flags_main,loan_dist_cheb_main,ubound_cheb_main,lbound_cheb_main,Hdot_main]=...
                run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);

%PF RUN
fore_disc=.645;
[P,P_pf,W_pf,I,Fore_rate,W,m_phi,mu_true,mu_rational,flags,loan_dist_cheb,ubound_cheb,Hdot]=...
    run_model_smm_pf(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),fore_disc,cost_flag);


Time = [0:1/52:T];
close all;

subplot(1,2,1), plot(Time,P_main./P_main(1),'LineWidth',1.5);
hold on;
subplot(1,2,1), plot(Time,P_pf./P_pf(1),'LineWidth',1.5,'LineStyle','--');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
h = legend('Baseline','Lender perfect foresight','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');
title('Price Index','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
subplot(1,2,2), plot(Time,W_main./P_main,'LineWidth',1.5);
hold on;
subplot(1,2,2), plot(Time,W_pf./P_pf,'LineWidth',1.5,'LineStyle','--');
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Mortgage Points','Interpreter','latex','FontSize',10)
xlabel('Years','Interpreter','latex')
set(gcf, 'Units', 'Inches', 'Position', [0, 0, 8, 2.75], 'PaperUnits', 'Inches', 'PaperSize', [8,3])
saveas(gcf,'output/slides_lenderforesight.pdf')

