% Load CBSA moments
clear;

addpath(genpath('/projectnb/gurengrp2/CRGM/SMM/shared_libraries'))
addpath(genpath('/Users/adam/Dropbox/Research/House Prices Growth Misperception (CRG)/Code'))

% Setup exogenous params
gam=0;
iota_tot = .045;
theta=0.005;
%mubar=0.010694293; 
mubar=.01496088; % Quartile 4
sigma_mu=.001;
LTV_load_flag = 'on';
save('exog_params','gam','iota_tot','theta','sigma_mu','LTV_load_flag');
T = 25;
zeta=0;
iota_R = 0;
nu = theta;
k=8;
aa=1;
cost_flag = 1;

% Load Moments
data= readtable('../data_moments/cbsaquantile4_data-weighted.csv');
for q=1:4

    eval(['boomstart = find(strcmp(data.quarterly,data.Q',num2str(q),'boomstart));'])
    eval(['peak = find(strcmp(data.quarterly,data.Q',num2str(q),'boomend));'])
    eval(['trough = find(strcmp(data.quarterly,data.Q',num2str(q),'bustend));'])
    boomstartcommon = find(strcmp(data.quarterly,'1998q1'));
    sampleend = find(strcmp(data.quarterly,'2019q4'));
    eval(['lhpi = log(data.Q',num2str(q),'HPIFM);']);
    eval(['H = data.Q',num2str(q),'unitsH;']);
    I = log(H./lagmatrix(H,4)) .* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since units are annual
    eval(['I = data.Q',num2str(q),'permitsI;']); %Use permits
    I = I./H; %Permits are Hdot
    I = I.* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since permits are annual
    I(I==0)=NaN;
    eval(['q',num2str(q),'hpi=data.Q',num2str(q),'HPIFM;']);

    boomsize_moment(q) = lhpi(peak) - lhpi(boomstart);
    bustsize_moment(q) = -1*[lhpi(trough) - lhpi(peak)];
    reboundsize_moment(q) = lhpi(trough+28) - lhpi(trough);
    bbrgrowth_moment(q) = (lhpi(sampleend) - lhpi(boomstartcommon))/((sampleend-(boomstartcommon))/4);
    boomconstruction_moment(q) = max(I(boomstartcommon:peak))/mean(I(boomstartcommon-20:boomstartcommon-4),'omitnan');
    bustconstruction_moment(q) = min(I(peak:trough+28))/max(I(boomstartcommon:peak));
    std_dP_bust_moment(q) = std(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    kurtosis_dP_bust_moment(q) = kurtosis(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    maxdprice_bust_moment(q) = min(lhpi(peak:trough) - lhpi(peak-4:trough-4));
    maxdprice_rebound_moment(q) = max(lhpi(trough:trough+28) - lhpi(trough-4:trough+24));
    eval(['boomlength_moment(q) = data.Q',num2str(q),'boomlength(1);'])
    eval(['bustlength_moment(q) = data.Q',num2str(q),'bustlength(1);'])
    eval(['maxforeclosure_moment(q) = data.Q',num2str(q),'max_foreclosure_rate(1)/100;']);
    eval(['yr9_negeq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt0(1);']);
    eval(['yr9_10eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt10(1);']);
    eval(['yr9_20eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt20(1);']);
    % No expectations moments by quartile.
    mubar(q) = (lhpi(88)-lhpi(9))/((88-9)/4);
end
eta_moment = 1./[data.Q1invelasticity(1),data.Q2invelasticity(1),data.Q3invelasticity(1),data.Q4invelasticity(1)];
expectation_2006_moment = .05;
expectation_2019_moment = .031;
bb_moment = [.09,.09,.09,.09];

% Load solver solution
load('optimal_params_xc.mat');

% run things
Time = [0:1/52:T];
moments = zeros(17,4);
sim_momments = zeros(17,4);

for q=1:4
%MAIN RUN
    [P,I,Fore_rate,~,m_phi,~,~,flags,loan_dist_cheb,ubound_cheb,lbound_cheb,Hdot,R10]=...
                run_model_smm_nopoints_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag);
  
    Fore_rate(end) = Fore_rate(end-1);
    mu_true = mu0(q)*exp(-theta*Time) + mubar(q)*(1-exp(-theta*Time));
    
    P_mat(:,q) = P;
    I_mat(:,q) = I;
    Fore_rate_mat(:,q) = Fore_rate;
    m_phi_mat(:,q) = m_phi;
    mu_true_mat(:,q) = mu_true;

    %{
    subplot(2,2,1), plot(Time,P./P(1),'LineWidth',1.5);
    title('Price Index')
    subplot(2,2,2), plot(Time,I./I(1),'LineWidth',1.5);
    title('Construction')
    subplot(2,2,3), plot(Time,Fore_rate,'LineWidth',1.5);
    title('Foreclosure Rate')
    subplot(2,2,4), plot(Time,m_phi,'LineWidth',1.5);
    hold on;
    subplot(2,2,4), plot(Time,mu_true,'LineStyle','--','Color','black');
    hold off;
    title('Beliefs')
    %}

    [boomsize(q),bustsize(q),reboundsize(q),bbrgrowth(q),boomlength(q),bustlength(q),maxforeclosure(q),std_dP_bust(q),kurtosis_dP_bust(q),maxdprice_bust(q),maxdprice_rebound(q)] = ...
        calc_moments(Time,P,Fore_rate)
    M_pdf=chebfun(loan_dist_cheb(:,10),[lbound_cheb,ubound_cheb],'coeffs'); %beginning of year 9 is the 10th one (start at zero)
    yr9_negeq(q) = sum(M_pdf([P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
    yr9_10eq(q) = sum(M_pdf([.9*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
    yr9_20eq(q) = sum(M_pdf([.8*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));

    % Expectations
    expectation_2006(q) = R10(1);
    expectation_2019(q) = R10(2);
    m_phi_2006(q) = m_phi(52*9+1);
    m_phi_2019(q) = m_phi(52*22+1);
    


    % Calculate SSR
    if q==4 
        moments(:,q) = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
                boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
                kurtosis_dP_bust_moment(q);maxdprice_bust_moment(q);...
                maxdprice_rebound_moment(q);bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);...
                expectation_2006_moment;expectation_2019_moment];
        weights=[0,0,0,1,0,1,1,0,1/2,1/2,0,1,1/3,1/3,1/3,1,1]; 
        
    else
        moments(:,q) = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
            boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
                kurtosis_dP_bust_moment(q);maxdprice_bust_moment(q);...
                maxdprice_rebound_moment(q);bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);1;1];
        weights=[0,0,0,1,0,1,1,0,1/2,1/2,0,1,1/3,1/3,1/3,0,0];
    end
    weights = weights/sum(weights);
    sim_moments(:,q) = [boomsize(q);bustsize(q);reboundsize(q);bbrgrowth(q);boomlength(q);bustlength(q);maxforeclosure(q);...
            std_dP_bust(q);kurtosis_dP_bust(q);maxdprice_bust(q);maxdprice_rebound(q);bb(q);...
yr9_negeq(q);yr9_10eq(q);yr9_20eq(q);m_phi_2006(q);m_phi_2019(q)];  
    ssr_sim(q) = weights*(((sim_moments(:,q)-moments(:,q))./moments(:,q)).^2)
end

format short
ssr
ssr_sim

sim_moments
moments
deviation = (sim_moments - moments)./moments

eta
mu0
mubar
mu0 - mubar
kappa
nu
k
phi
chi
iota_L
rho
aa
bb
gamma_elas
%exitflag_mat
format long

P_nopoints = P_mat;

load('optimal_params_xc.mat');
nTrials_mc = 100;

for q=1:4
%MAIN RUN
    [P,I,Fore_rate,~,m_phi,~,~,flags,loan_dist_cheb,ubound_cheb,lbound_cheb,Hdot,R10]=...
                run_model_smm_beliefs(mu0(q),mubar(q),sigma_mu,theta,phi(q),nu,k,kappa(q),chi(q),iota_L(q),iota_R,rho(q),eta(q),aa,bb(q),gamma_elas(q),cost_flag,nTrials_mc);
  
    Fore_rate(end) = Fore_rate(end-1);
    mu_true = mu0(q)*exp(-theta*Time) + mubar(q)*(1-exp(-theta*Time));
    
    P_mat(:,q) = P;
    I_mat(:,q) = I;
    Fore_rate_mat(:,q) = Fore_rate;
    m_phi_mat(:,q) = m_phi;
    mu_true_mat(:,q) = mu_true;

    %{
    subplot(2,2,1), plot(Time,P./P(1),'LineWidth',1.5);
    title('Price Index')
    subplot(2,2,2), plot(Time,I./I(1),'LineWidth',1.5);
    title('Construction')
    subplot(2,2,3), plot(Time,Fore_rate,'LineWidth',1.5);
    title('Foreclosure Rate')
    subplot(2,2,4), plot(Time,m_phi,'LineWidth',1.5);
    hold on;
    subplot(2,2,4), plot(Time,mu_true,'LineStyle','--','Color','black');
    hold off;
    title('Beliefs')
    %}

    [boomsize(q),bustsize(q),reboundsize(q),bbrgrowth(q),boomlength(q),bustlength(q),maxforeclosure(q),std_dP_bust(q),kurtosis_dP_bust(q),maxdprice_bust(q),maxdprice_rebound(q)] = ...
        calc_moments(Time,P,Fore_rate)
    M_pdf=chebfun(loan_dist_cheb(:,10),[lbound_cheb,ubound_cheb],'coeffs'); %beginning of year 9 is the 10th one (start at zero)
    yr9_negeq(q) = sum(M_pdf([P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
    yr9_10eq(q) = sum(M_pdf([.9*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
    yr9_20eq(q) = sum(M_pdf([.8*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));

    % Expectations
    expectation_2006(q) = R10(1);
    expectation_2019(q) = R10(2);
    m_phi_2006(q) = m_phi(52*9+1);
    m_phi_2019(q) = m_phi(52*22+1);
    


    % Calculate SSR
    if q==4 
        moments(:,q) = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
                boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
                kurtosis_dP_bust_moment(q);maxdprice_bust_moment(q);...
                maxdprice_rebound_moment(q);bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);...
                expectation_2006_moment;expectation_2019_moment];
        weights=[0,0,0,1,0,1,1,0,1/2,1/2,0,1,1/3,1/3,1/3,1,1]; 
        
    else
        moments(:,q) = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
            boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
                kurtosis_dP_bust_moment(q);maxdprice_bust_moment(q);...
                maxdprice_rebound_moment(q);bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);1;1];
        weights=[0,0,0,1,0,1,1,0,1/2,1/2,0,1,1/3,1/3,1/3,0,0];
    end
    weights = weights/sum(weights);
    sim_moments(:,q) = [boomsize(q);bustsize(q);reboundsize(q);bbrgrowth(q);boomlength(q);bustlength(q);maxforeclosure(q);...
            std_dP_bust(q);kurtosis_dP_bust(q);maxdprice_bust(q);maxdprice_rebound(q);bb(q);...
yr9_negeq(q);yr9_10eq(q);yr9_20eq(q);m_phi_2006(q);m_phi_2019(q)];  
    ssr_sim(q) = weights*(((sim_moments(:,q)-moments(:,q))./moments(:,q)).^2)
end

format short
ssr
ssr_sim

sim_moments
moments
deviation = (sim_moments - moments)./moments

eta
mu0
mubar
mu0 - mubar
kappa
nu
k
phi
chi
iota_L
rho
aa
bb
gamma_elas
%exitflag_mat
format long

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Plot of best fit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
year = [1975:.25:2020.75];
yrs = find(year >= 1998 & year <= 2020);

subplot(2,2,1), plot(Time,P_nopoints(:,1)./P_nopoints(1,1),'LineWidth',1.5);
hold on;
subplot(2,2,1), plot(Time,P_mat(:,1)./P_mat(1,1),'LineWidth',1.5);
subplot(2,2,1), plot([0:.25:22],q1hpi(yrs)./q1hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--')
%subplot(2,2,1), text(7,2.25,['$\mu_0$ = ',num2str(mu0(1)),'\%, $\chi$ = ',num2str(chi(1)),'\%, $\iota$ =', num2str(iota_L(1))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
h = legend('Model Nopoints','Model Points','Data','Interpreter','latex','Box','off');
h.FontSize = 7;
set(h,'Location','best');
title('Quartile 1 (Smallest)','Interpreter','latex','FontSize',10);
ylim([.5,3]);
ylabel('Price Index','Interpreter','latex');

subplot(2,2,2), plot(Time,P_nopoints(:,2)./P_nopoints(1,2),'LineWidth',1.5);
hold on;
subplot(2,2,2), plot(Time,P_mat(:,2)./P_mat(1,2),'LineWidth',1.5);
subplot(2,2,2), plot([0:.25:22],q2hpi(yrs)./q2hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--');
%subplot(2,2,2), text(7,2.25,['$\mu_0$ = ',num2str(mu0(2)),'\%, $\chi$ = ',num2str(chi(2)),'\%, $\iota$ =', num2str(iota_L(2))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Quartile 2','Interpreter','latex','FontSize',10);
ylim([.9,3]);
ylabel('Price Index','Interpreter','latex');

subplot(2,2,3), plot(Time,P_nopoints(:,3)./P_nopoints(1,3),'LineWidth',1.5);
hold on;
subplot(2,2,3), plot(Time,P_mat(:,3)./P_mat(1,3),'LineWidth',1.5);
subplot(2,2,3), plot([0:.25:22],q3hpi(yrs)./q3hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--');
%subplot(2,2,3), text(7,2.25,['$\mu_0$ = ',num2str(mu0(3)),'\%, $\chi$ = ',num2str(chi(3)),'\%, $\iota$ =', num2str(iota_L(3))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Quartile 3','Interpreter','latex','FontSize',10);
ylim([.9,3]);
xlabel('Years','Interpreter','latex');
ylabel('Price Index','Interpreter','latex');

subplot(2,2,4), plot(Time,P_nopoints(:,4)./P_nopoints(1,4),'LineWidth',1.5);
hold on;
subplot(2,2,4), plot(Time,P_mat(:,4)./P_mat(1,4),'LineWidth',1.5);
subplot(2,2,4), plot([0:.25:22],q4hpi(yrs)./q4hpi(min(yrs)),'LineWidth',1.5,'LineStyle','--');
%subplot(2,2,4), text(7,1.1,['$\mu_0$ = ',num2str(mu0(4)),'\%, $\chi$ = ',num2str(chi(4)),'\%, $\iota$ =', num2str(iota_L(4))],'Interpreter','latex','FontSize',8);
axis([0 T -inf inf])
xticks([0 5 10 15 20 25]);
hold off;
title('Quartile 4','Interpreter','latex','FontSize',10);
ylim([.9,3]);
xlabel('Years','Interpreter','latex');
ylabel('Price Index','Interpreter','latex');
saveas(gcf,'output/pointscompare.pdf')
