function [R1,R10,P1,P10,Pf] = priceBeliefsCalcFull(X_eq,I_eq,Ibar_eq,a_eq,F_eq,P_eq,L,U,PP,M_nodes,ff,bW,...
                           mubar,gamma_elas,alpha_Ibar,sigma_D,iota_L,iota_R,u_bound_integral,...
                           theta,sigma_mu,sigma_m,N0,gH,c,eta,chi,xm,l_bound,u_bound,rho,chebsize,J,LTV_dist,gam,Tf,cost_flag)
%UNTITLED2 Summary of this function goes here
%   Detailed explanation goes here

close all;
gamma=gamma_elas;
alpha=alpha_Ibar;
sigma=sigma_D;
iota=iota_L+iota_R;
u_bound_int=u_bound_integral;
options = optimoptions('fsolve','Display','off','SpecifyObjectiveGradient',true);


T=Tf;
dt=1/ff; %Simulation stepsize
nPeriods=T/dt; %Number of simulation periods
TT=0:1/ff:T;

N=size(X_eq,1);
Pf=zeros(N,nPeriods+1);
P1=zeros(N,1);
P10=zeros(N,1);
R1=zeros(N,1);
R10=zeros(N,1);

D_eq=X_eq(:,1);
mu_eq=X_eq(:,2);
H_eq=X_eq(:,3);

T_set=1:N;
parfor tt=1:size(T_set,2)
    tt
    D0=D_eq(tt);
    mu0=mu_eq(tt);
    H0=H_eq(tt);

    X=zeros(nPeriods+1,3);
    mu_true=mubar+(mu0-mubar)*exp(-theta*TT);
    X(:,1) = D0*exp(mubar*TT+...
            1/theta*(1-exp(-theta*TT))*(mu0-mubar)+...
            sigma_mu^2/2/theta^2*TT-3*sigma_mu^2/4/theta^3+...
            sigma_mu^2/4/theta^3*(4*exp(-theta*TT)-exp(-2*theta*TT)));
    X(:,2)=mu_true;
    V = PV(X(:,2),rho,mubar,theta,sigma_mu,sigma_m,X(:,1));

    P=zeros(nPeriods+1,1);
    P_init=zeros(nPeriods+1,1);
    I=zeros(nPeriods+1,1);
    Ibar=zeros(nPeriods+1,1);
    W=zeros(nPeriods+1,1);
    Fore_rate=zeros(nPeriods+1,1);
    F=zeros(nPeriods+1,1);
    flags=zeros(nPeriods+1,1);
    Change=zeros(chebsize+1,nPeriods+1);
    Add=zeros(chebsize+1,nPeriods+1);
    a=zeros(chebsize+1,nPeriods+1);

    X(1,3)=H0;
    I(1)=I_eq(tt);
    Ibar(1)=Ibar_eq(tt);
    F(1)=F_eq(tt);
    W(1)=[X(1,:) Ibar(1) 1]*bW;
    P_init(1)=P_eq(tt);

    a(:,1)=a_eq(:,tt);
    M_pdf=chebfun(a(:,1),[l_bound,u_bound],'coeffs');
    M_diff=diff(M_pdf);

    for t=2:nPeriods+1
            
           %Solve for equilibrium price and investment
           [P(t-1),I(t-1),flags(t-1)]...
               =Solve_Eq_Price_Impulse(V(t-1),X(t-1,3),N0,gH,c,eta,chi,xm,gamma,Ibar(t-1),W(t-1),F(t-1),1,P_init(t-1),cost_flag,options);
           
           P_init(t)=P(t-1);

           %Update housing stock
           X(t,3)=X(t-1,3)+I(t-1)*X(t-1,3)*dt;

           %Update investment moving average
           Ibar(t) = (1-alpha)*Ibar(t-1)+alpha*I(t-1);

           %Determine equilibrium points and price forecasts
           W(t)=[X(t,:) Ibar(t) 1]*bW;

           %Update foreclosures for period t and determine foreclosure rate
           F(t)= iota_L*integral(@(M) M_pdf(M).*J(M./P(t-1)),l_bound,u_bound_int);
           Fore_rate(t-1)=F(t)/X(t-1,3);

           %Update loan balance distribution
           Change_vec=gam*(M_pdf(M_nodes)+M_nodes.*M_diff(M_nodes))...
                  +(I(t-1)*X(t-1,3)+F(t-1))*LTV_dist(M_nodes/P(t-1))/P(t-1)...
                  -iota*M_pdf(M_nodes)+(iota*X(t-1,3)-F(t))*LTV_dist(M_nodes/P(t-1))/P(t-1);

           Change(:,t)=Change_vec;
           Change(1,t)=0;
           Change(end,t)=0;

           Add(:,t)=U\(L\(PP*Change(:,t)*dt));
           a(:,t)=a(:,t-1)+Add(:,t);

           M_pdf=chebfun(a(:,t),[l_bound,u_bound],'coeffs');
           M_diff=diff(M_pdf);



    end
    
    P(end)=P(end-1);
    Pf(tt,:)=P;
    P1(tt)=P(1/dt);
    P10(tt)=P(10/dt);
    R1(tt)=P(1/dt)/P(1)-1;
    R10(tt)=(P(10/dt)/P(1))^(1/10)-1;
end

end

