function run_smm(nBlocks,blockNum)
    % Runs smm in nBlocks blocks and does so for block number blockNum
    addpath(genpath('/projectnb/gurengrp2/CRGM/SMM/shared_libraries'))

    % exogenous params
    gam=0;
    iota_tot = .045;
    theta=0.005;
    mubar=.01; 
    sigma_mu=.001;
    LTV_load_flag = 'on';
    save('exog_params','gam','iota_tot','theta','mubar','sigma_mu','LTV_load_flag');
   
    % Finer Window Grid 7/27/2021
    eta_vec = 1./[1.0138328];
    mu0_vec=[.021:.001:.027]; %rebound size
    kappa_vec=[.11:.01:.17]; % bust length
    k_vec = [8]; 
    phi_vec=[3.75:.125:4.5]; % boom size and bust size
    chi_vec = [.03:.00125:.0375]; % smoothness of bust
    iota_L_vec = [.045:.0025:.0525];
    rho_vec = [.055:.0025:.075]; % boom size
    aa_vec=[1];
    bb_vec=[.06:.01:.11]; %smootheness
    gamma_elas_vec=[0.95:0.05:1.15]; % Price impact of foreclosures
   
    nparams = length(eta_vec)*length(mu0_vec)*length(kappa_vec)*length(k_vec)*length(phi_vec)*length(chi_vec)*length(iota_L_vec)*length(rho_vec)*length(aa_vec)*length(bb_vec)*length(gamma_elas_vec)
    years = 25; 

    % Set up outputting matrices
    pers = years+1;
    pers_vec = [1:52:(52*years+1)];
    Time=[0:1/52:years];
    
    P_results_mat = zeros(pers,nparams);
    %Fore_rate_results_mat = zeros(pers,nparams);
    %I_results_mat = zeros(pers,nparams);
    %W_results_mat = zeros(pers,nparams);
    %m_phi_results_mat = zeros(pers,nparams);
    
    eta_results_mat = zeros(1,nparams);
    mu0_results_mat = zeros(1,nparams);
    kappa_results_mat = zeros(1,nparams);
    nu_results_mat = zeros(1,nparams);
    k_results_mat = zeros(1,nparams);
    phi_results_mat = zeros(1,nparams);
    chi_results_mat = zeros(1,nparams);
    iota_L_results_mat = zeros(1,nparams);
    rho_results_mat=zeros(1,nparams);
    aa_results_mat = zeros(1,nparams);
    bb_results_mat = zeros(1,nparams);
    gamma_elas_results_mat = zeros(1,nparams);
    
    flags_results_mat = zeros(1,nparams);
    boomsize_results_mat = zeros(1,nparams);
    bustsize_results_mat = zeros(1,nparams);
    reboundsize_results_mat = zeros(1,nparams);
    boomlength_results_mat = zeros(1,nparams);
    bustlength_results_mat = zeros(1,nparams);
    maxforeclosure_results_mat = zeros(1,nparams);
    maxdprice_bust_results_mat = zeros(1,nparams);
    maxdprice_rebound_results_mat = zeros(1,nparams);
    yr9_negeq_results_mat = zeros(1,nparams);
    yr9_10eq_results_mat = zeros(1,nparams);
    yr9_20eq_results_mat = zeros(1,nparams);
    bbrgrowth_results_mat = zeros(1,nparams);
    expectation_2006_results_mat = zeros(1,nparams);
    expectation_2019_results_mat = zeros(1,nparams);
    
    perblock = ceil(nparams/nBlocks);
    startnum = (blockNum-1)*perblock+1
    endnum = min(blockNum*perblock,nparams)
    
    % SIMNUMBER
    tic
    for simnumber = startnum:endnum
        display('Solving Simulation number  '+string(simnumber))

        [eta_ind,mu0_ind,kappa_ind,k_ind,phi_ind,chi_ind,iota_L_ind,rho_ind,aa_ind,bb_ind,gamma_elas_ind] =...
            ind2sub([length(eta_vec) length(mu0_vec) length(kappa_vec) length(k_vec) length(phi_vec) length(chi_vec) length(iota_L_vec) length(rho_vec) length(aa_vec) length(bb_vec) length(gamma_elas_vec)],simnumber);
        eta = eta_vec(eta_ind);
        mu0 = mu0_vec(mu0_ind);
        kappa = kappa_vec(kappa_ind);
        k = k_vec(k_ind);
        nu = theta; % fix nu = theta
        phi = phi_vec(phi_ind);
        chi = chi_vec(chi_ind);
        iota_L = iota_L_vec(iota_L_ind);
        iota_R = max(iota_tot - iota_L,0);
        rho = rho_vec(rho_ind);
        aa = aa_vec(aa_ind);
        bb = bb_vec(bb_ind);
        gamma_elas = gamma_elas_vec(gamma_elas_ind);

        try
            [P,~,Fore_rate,~,~,~,~,flags,loan_dist_cheb,ubound_cheb,lbound_cheb,~,~,~,~,R10]=...
                run_model_smm_beliefs(mu0,mubar,sigma_mu,theta,phi,nu,k,kappa,chi,iota_L,iota_R,rho,eta,aa,bb,gamma_elas);
            
            P_results_mat(:,simnumber)=P(pers_vec);
            %Fore_rate_results_mat(:,simnumber)=Fore_rate(pers_vec);
            %W_results_mat(:,simnumber)=W(pers_vec);
            %I_results_mat(:,simnumber) = I(pers_vec);
            %m_phi_results_mat(:,simnumber)=m_phi(pers_vec);
            flags_results_mat(simnumber) = min(flags(1:end-1));
            
            [boomsize_results_mat(simnumber),bustsize_results_mat(simnumber),reboundsize_results_mat(simnumber),bbrgrowth_results_mat(simnumber),...
                boomlength_results_mat(simnumber),bustlength_results_mat(simnumber),maxforeclosure_results_mat(simnumber),...
                maxdprice_bust_results_mat(simnumber),maxdprice_rebound_results_mat(simnumber)] = ...
                calc_moments(Time,P,Fore_rate);
            
             % Loan balance dist in year 9
            M_pdf=chebfun(loan_dist_cheb(:,10),[lbound_cheb,ubound_cheb],'coeffs'); %beginning of year 9 is the 10th one (start at zero)
            yr9_negeq_results_mat(simnumber) = sum(M_pdf([P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
            yr9_10eq_results_mat(simnumber) = sum(M_pdf([.9*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
            yr9_20eq_results_mat(simnumber) = sum(M_pdf([.8*P(52*9+1):.001:ubound_cheb]))/sum(M_pdf([lbound_cheb:.001:ubound_cheb]));
            
            % Expectations
            expectation_2006_results_mat(simnumber) = R10(1);
            expectation_2019_results_mat(simnumber) = R10(2);
        catch
            display('ERROR IN SIMULATION NUMBER '+string(simnumber))
            P_results_mat(:,simnumber)=nan;
            %Fore_rate_results_mat(:,simnumber)=nan;
            %W_results_mat(:,simnumber)=nan;
            %I_results_mat(:,simnumber)=nan;
            %m_phi_results_mat(:,simnumber)=nan;
            flags_results_mat(simnumber)=nan;
            
            boomsize_results_mat(simnumber) = nan;
            bustsize_results_mat(simnumber) = nan;
            reboundsize_results_mat(simnumber) = nan;
            boomlength_results_mat(simnumber) = nan;
            bustlength_results_mat(simnumber) = nan;
            maxforeclosure_results_mat(simnumber) = nan;
            maxdprice_bust_results_mat(simnumber) = nan;
            maxdprice_rebound_results_mat(simnumber) = nan;
            yr9_negeq_results_mat(simnumber) = nan;
            yr9_10eq_results_mat(simnumber) = nan;
            yr9_20eq_results_mat(simnumber) = nan;
            bbrgrowth_results_mat(simnumber) = nan;
            expectation_2006_results_mat(simnumber) = nan;
            expectation_2019_results_mat(simnumber) = nan;
        end
        eta_results_mat(simnumber)=eta;
        mu0_results_mat(simnumber)=mu0;
        kappa_results_mat(simnumber)=kappa;
        nu_results_mat(simnumber)=nu;
        k_results_mat(simnumber)=k;
        phi_results_mat(simnumber)=phi;
        chi_results_mat(simnumber)=chi;
        iota_L_results_mat(simnumber)=iota_L;
        rho_results_mat(simnumber)=rho;
        aa_results_mat(simnumber)=aa;
        bb_results_mat(simnumber)=bb;
        gamma_elas_results_mat(simnumber)=gamma_elas;
    end
    toc
    save(strcat('smm_block',string(blockNum),'.mat'),'P_results_mat','flags_results_mat',...
            'boomsize_results_mat','bustsize_results_mat','reboundsize_results_mat',...
            'boomlength_results_mat','bustlength_results_mat','maxforeclosure_results_mat',...
            'maxdprice_bust_results_mat','maxdprice_rebound_results_mat',...
            'yr9_negeq_results_mat','yr9_10eq_results_mat','yr9_20eq_results_mat',...
            'bbrgrowth_results_mat','expectation_2006_results_mat','expectation_2019_results_mat',...
            'eta_results_mat','mu0_results_mat','kappa_results_mat',...
            'nu_results_mat','k_results_mat','phi_results_mat','chi_results_mat',...
            'iota_L_results_mat','rho_results_mat','aa_results_mat','bb_results_mat',...
            'gamma_elas_results_mat','-v7.3');
             %'Fore_rate_results_mat','m_phi_results_mat','I_results_mat',...
            %'W_results_mat',...
end
