clear
load smm_combined.mat;
addpath('../shared_libraries/chebfun');

T=25;
ff=4; % even though we simulate 52 weeks we only save annual in the loop

%Initialization
Time=[1/ff:1/ff:T];
nparams=length(chi_mat);

% Target moments come from quartile 3 of data
q = 4;
data= readtable('../data_moments/cbsaquantile4_data-weighted.csv');

eval(['boomstart = find(strcmp(data.quarterly,data.Q',num2str(q),'boomstart));'])
eval(['peak = find(strcmp(data.quarterly,data.Q',num2str(q),'boomend));'])
eval(['trough = find(strcmp(data.quarterly,data.Q',num2str(q),'bustend));'])
boomstartcommon = find(strcmp(data.quarterly,'1998q1'));
sampleend = find(strcmp(data.quarterly,'2019q4'));
eval(['lhpi = log(data.Q',num2str(q),'HPIFM);']);
eval(['H = data.Q',num2str(q),'unitsH;']);
I = log(H./lagmatrix(H,4)) .* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since units are annual
eval(['I = data.Q',num2str(q),'permitsI;']); %Use permits
I = I./H; %Permits are Hdot
I = I.* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since permits are annual
I(I==0)=NaN;
eval(['q',num2str(q),'hpi=data.Q',num2str(q),'HPIFM;']);

boomsize_moment = lhpi(peak) - lhpi(boomstart);
bustsize_moment = -1*[lhpi(trough) - lhpi(peak)];
reboundsize_moment = lhpi(trough+28) - lhpi(trough);
bbrgrowth_moment = (lhpi(sampleend) - lhpi(boomstartcommon))/((sampleend-boomstartcommon)/4);
sampleend_H = 177;
bbrgrowth_H_moment = (log(H(sampleend_H)) - log(H(boomstartcommon)))/((sampleend_H-boomstartcommon)/4);
maxdprice_bust_moment = min(lhpi(peak:trough) - lhpi(peak-4:trough-4));
maxdprice_rebound_moment = max(lhpi(trough:trough+28) - lhpi(trough-4:trough+24));
eval(['boomlength_moment = data.Q',num2str(q),'boomlength(1);']);
eval(['bustlength_moment = data.Q',num2str(q),'bustlength(1);']);
eval(['maxforeclosure_moment = data.Q',num2str(q),'max_foreclosure_rate(1)/100;']);
eval(['yr9_negeq_moment = data.Q',num2str(q),'jan2007equitysharelt0(1);']);
eval(['yr9_10eq_moment = data.Q',num2str(q),'jan2007equitysharelt10(1);']);
eval(['yr9_20eq_moment = data.Q',num2str(q),'jan2007equitysharelt20(1);']);
expectation_2006_moment = .05;
expectation_2019_moment = .031;
bb_moment = .09;
std_dP_bust_moment = std(lhpi(peak+4:trough)-lhpi(peak:trough-4));
kurtotis_dP_bust_moment = kurtosis(lhpi(peak+4:trough)-lhpi(peak:trough-4));

moments_vec = [boomsize_moment;bustsize_moment;reboundsize_moment;bbrgrowth_moment;boomlength_moment;bustlength_moment;maxforeclosure_moment;std_dP_bust_moment;...
    kurtotis_dP_bust_moment;maxdprice_bust_moment;maxdprice_rebound_moment;bb_moment;yr9_negeq_moment;yr9_10eq_moment;yr9_20eq_moment;expectation_2006_moment;expectation_2019_moment];

%Weight vector
weights=[0,0,0,1,0,1,1,0,0,1,0,1,1/3,1/3,1/3,1,1]; %No weight on boom length because it is covered by k
weights = weights/sum(weights);
sim_mat = [boomsize_mat;bustsize_mat;reboundsize_mat;bbrgrowth_mat;boomlength_mat;bustlength_mat;maxforeclosure_mat;std_dP_bust_mat;kurtosis_dP_bust_mat;...
    maxdprice_bust_mat;maxdprice_rebound_mat;bb_mat;yr9_negeq_mat;yr9_10eq_mat;yr9_20eq_mat;expectation_2006_mat;expectation_2019_mat];
moments_mat = repmat(moments_vec,1,nparams);

% Calculate sum of squared errors and find minimum
weighted_ssr = weights*(((sim_mat-moments_mat)./moments_mat).^2);

[min_ssr, index_min] = min(weighted_ssr)

% Minimum of moments, targets, and deviation
simulated_moments = sim_mat(:,index_min)
actual_moments = moments_vec
percent_deviation = (sim_mat(:,index_min)-moments_vec)./moments_vec
ssr = sum(weights*percent_deviation.^2)

%actual parametrization
eta = eta_mat(index_min)
mu0 = mu0_mat(index_min)
kappa = kappa_mat(index_min)
%nu = nu_mat(index_min)
k = k_mat(index_min)
phi = phi_mat(index_min)
chi = chi_mat(index_min)
iota_L = iota_L_mat(index_min)
rho = rho_mat(index_min)
%aa = aa_mat(index_min)
bb = bb_mat(index_min)
gamma_elas = gamma_elas_mat(index_min)
%flags = flags_mat(index_min)

%plot of best fit
plot(Time,P_mat(:,index_min),'LineWidth',1.5);
title('P')
save('optimal_params.mat','eta','mu0','kappa','k','phi','chi','iota_L','rho','bb','gamma_elas','simulated_moments');
