% Load CBSA moments
clear;

load smm_combined_xc_q1.mat;
load optimal_params_xc.mat

data= readtable('../data_moments/cbsaquantile4_data-weighted.csv');
for q=1:4

    eval(['boomstart = find(strcmp(data.quarterly,data.Q',num2str(q),'boomstart));'])
    eval(['peak = find(strcmp(data.quarterly,data.Q',num2str(q),'boomend));'])
    eval(['trough = find(strcmp(data.quarterly,data.Q',num2str(q),'bustend));'])
    boomstartcommon = find(strcmp(data.quarterly,'1998q1'));
    sampleend = find(strcmp(data.quarterly,'2019q4'));
    eval(['lhpi = log(data.Q',num2str(q),'HPIFM);']);
    eval(['H = data.Q',num2str(q),'unitsH;']);
    I = log(H./lagmatrix(H,4)) .* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since units are annual
    eval(['I = data.Q',num2str(q),'permitsI;']); %Use permits
    I = I./H; %Permits are Hdot
    I = I.* cellfun(@(x) isnumeric(x) && numel(x)==1,(regexp(data.quarterly,'[12][9012][0-9][0-9]q1'))); %Since permits are annual
    I(I==0)=NaN;
    eval(['q',num2str(q),'hpi=data.Q',num2str(q),'HPIFM;']);

    boomsize_moment(q) = lhpi(peak) - lhpi(boomstart);
    bustsize_moment(q) = -1*[lhpi(trough) - lhpi(peak)];
    reboundsize_moment(q) = lhpi(trough+28) - lhpi(trough);
    bbrgrowth_moment(q) = (lhpi(sampleend) - lhpi(boomstartcommon))/((sampleend-(boomstartcommon))/4);
    boomconstruction_moment(q) = max(I(boomstartcommon:peak))/mean(I(boomstartcommon-20:boomstartcommon-4),'omitnan');
    bustconstruction_moment(q) = min(I(peak:trough+28))/max(I(boomstartcommon:peak));
    std_dP_bust_moment(q) = std(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    kurtotis_dP_bust_moment(q) = kurtosis(lhpi(peak+4:trough)-lhpi(peak:trough-4));
    maxdprice_bust_moment(q) = min(lhpi(peak:trough) - lhpi(peak-4:trough-4));
    maxdprice_rebound_moment(q) = max(lhpi(trough:trough+28) - lhpi(trough-4:trough+24));
    eval(['boomlength_moment(q) = data.Q',num2str(q),'boomlength(1);'])
    eval(['bustlength_moment(q) = data.Q',num2str(q),'bustlength(1);'])
    eval(['maxforeclosure_moment(q) = data.Q',num2str(q),'max_foreclosure_rate(1)/100;']);
    eval(['yr9_negeq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt0(1);']);
    eval(['yr9_10eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt10(1);']);
    eval(['yr9_20eq_moment(q) = data.Q',num2str(q),'jan2007equitysharelt20(1);']);
    % No expectations moments by quartile.
    mubar_vec(q) = (lhpi(88)-lhpi(9))/((88-9)/4);
end
eta_moment = 1./[data.Q1invelasticity(1),data.Q2invelasticity(1),data.Q3invelasticity(1),data.Q4invelasticity(1)];
expectation_2006_moment = .05;
expectation_2019_moment = .031;
bb_moment = [.09,.09,.09,.09];

addpath('../shared_libraries/chebfun');

T=25;
ff=4;

%Initialization
Time=[0:1/ff:T];
nparams=length(chi_mat);

boomsize=boomsize_mat;
bustsize=bustsize_mat;
reboundsize=reboundsize_mat;
bbrgrowth=bbrgrowth_mat;
boomlength=boomlength_mat;
bustlength=bustlength_mat;
maxforeclosure=maxforeclosure_mat;
std_dP_bust=std_dP_bust_mat;
kurtosis_dP_bust=kurtosis_dP_bust_mat;

%Weight vector

%Setup
actual_moments = simulated_moments;

% Target moments
for q = 1
    display(['Quantile ',num2str(q)])
    
    % Weights reset
    weights=  [0,0,0,bbrweight(q),0,1,1,0,0,1,0,1,1/3,1/3,1/3];
    weightsq4=[0,0,0,bbrweight(q),0,1,1,0,0,1,0,1,1/3,1/3,1/3,1,1];
    weights = weights/sum(weights);
    weightsq4 = weightsq4/sum(weightsq4);

    if q == 4 
        moments_vec = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
            boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
            kurtotis_dP_bust_moment(q);maxdprice_bust_moment(q);maxdprice_rebound_moment(q);...
            bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q);...
            expectation_2006_moment;expectation_2019_moment];
        sim_mat = [boomsize;bustsize;reboundsize;bbrgrowth;boomlength;bustlength;maxforeclosure;...
            std_dP_bust;kurtosis_dP_bust;maxdprice_bust_mat;maxdprice_rebound_mat;bb_mat;yr9_negeq_mat;yr9_10eq_mat;yr9_20eq_mat;expectation_2006_mat;expectation_2019_mat];
        moments_mat = repmat(moments_vec,1,nparams);
        % Calculate sum of squared errors and find minimum
        weighted_ssr = weightsq4*(((sim_mat-moments_mat)./moments_mat).^2);
    else
        moments_vec = [boomsize_moment(q);bustsize_moment(q);reboundsize_moment(q);bbrgrowth_moment(q);...
            boomlength_moment(q);bustlength_moment(q);maxforeclosure_moment(q);std_dP_bust_moment(q);...
            kurtotis_dP_bust_moment(q);maxdprice_bust_moment(q);maxdprice_rebound_moment(q);...
            bb_moment(q);yr9_negeq_moment(q);yr9_10eq_moment(q);yr9_20eq_moment(q)];
        sim_mat = [boomsize;bustsize;reboundsize;bbrgrowth;boomlength;bustlength;maxforeclosure;...
            std_dP_bust;kurtosis_dP_bust;maxdprice_bust_mat;maxdprice_rebound_mat;bb_mat;yr9_negeq_mat;yr9_10eq_mat;yr9_20eq_mat];
        moments_mat = repmat(moments_vec,1,nparams);
        % Calculate sum of squared errors and find minimum
        weighted_ssr = weights*(((sim_mat-moments_mat)./moments_mat).^2);
    end
  
    % Find minimum
    indices = find(eta_mat==eta_moment(q));
    [min_ssr index_min] = min(weighted_ssr(indices));
    index_min = indices(index_min);
    indices_min(q) = index_min;
    

    % If BBR deviation is too high, raise bbr weight until okay
    while abs((sim_mat(4,index_min)-moments_vec(4))./sim_mat(4,index_min)) > .1
        %increase bbrweight
        bbrweight(q) = bbrweight(q) + .1;
        
        %recalc weighted ssr
        if q == 4
            weightsq4=[0,0,0,bbrweight(q),0,1,1,0,0,1,0,1,1/3,1/3,1/3,1,1];
            weightsq4 = weightsq4/sum(weightsq4);
            weighted_ssr = weightsq4*(((sim_mat-moments_mat)./moments_mat).^2);
        else
            weights=  [0,0,0,bbrweight(q),0,1,1,0,0,1,0,1,1/3,1/3,1/3];
            weights = weights/sum(weights);
            weighted_ssr = weights*(((sim_mat-moments_mat)./moments_mat).^2);
        end
        
        % Find minimum
        indices = find(eta_mat==eta_moment(q));
        [min_ssr index_min] = min(weighted_ssr(indices));
        index_min = indices(index_min);
        indices_min(q) = index_min;
    end
    
    % Minimum of moments, targets, and deviation
    simulated_moments(:,q) = sim_mat(1:14,index_min);
    actual_moments(:,q) = moments_vec(1:14);
    percent_deviation = (simulated_moments(:,q)-actual_moments(:,q))./simulated_moments(:,q)
    ssr(q) = weighted_ssr(index_min);

    %actual parametrization
    eta(q) = eta_mat(index_min);
    mu0(q) = mu0_mat(index_min);
    kappa(q)  = kappa_mat(index_min);
    nu(q) = nu_mat(index_min);
    k(q) = k_mat(index_min)
    phi(q) = phi_mat(index_min);
    chi(q) = chi_mat(index_min);
    iota_L(q) = iota_L_mat(index_min);
    rho(q) = rho_mat(index_min);
    aa(q) = aa_mat(index_min);
    bb(q) = bb_mat(index_min);
    gamma_elas(q) = gamma_elas_mat(index_min);
    flags(q) = flags_mat(index_min);
    
    %Save beginning-of-year price
    P(:,q) = P_mat(:,index_min);
end

simulated_moments
actual_moments
(simulated_moments - actual_moments)./actual_moments
ssr


eta
mu0
mubar_vec
mu0 - mubar_vec
kappa
nu
k
phi
chi
iota_L
rho
aa
bb
gamma_elas
flags
bbrweight

% SAVE FOR FINAL RUN WITH POINTS
save('optimal_params_xc.mat','eta','mu0','kappa','phi','chi','iota_L','rho','bb','gamma_elas','bbrweight','simulated_moments','ssr');
