clear;
a = 0.5; 
b = 2; 
m = 30;
h = (b-a)/(m+1);
x = linspace(a,b,m+2);   % grid points x including boundaries
y = linspace(a,b,m+2);   % grid points y including boundaries
z = linspace(a,b,m+2);
tol=1e-7;

r=-0.05;
g=.02;
sig=.4;
nu1=.5;
nu2=.5;
nu3=.5;

[X,Y,Z] = meshgrid(x,y,z);      % 2d arrays of x,y values
X = permute(X,[2,1,3]);                     % transpose so that X(i,j),Y(i,j) are
Y = permute(Y,[2,1,3]);                     % coordinates of (i,j) point

Iint = 2:m+1;              % indices of interior points in x
Jint = 2:m+1;              % indices of interior points in y
Kint = 2:m+1;
Xint = X(Iint,Jint,Kint);       % interior points
Yint = Y(Iint,Jint,Kint);
Zint = Z(Iint,Jint,Kint);

Cxx=0.5*sig^2*X(Iint,Jint,Kint).^2;
Cyy=0.5*sig^2*Y(Iint,Jint,Kint).^2;
Czz=0.5*sig^2*Z(Iint,Jint,Kint).^2;
Cx=g*X(Iint,Jint,Kint);
Cy=g*Y(Iint,Jint,Kint);
Cz=g*Z(Iint,Jint,Kint);
C0=r*ones(m,m,m);

soln=@(x,y,z) x^nu1*y^nu2*z^nu3;
soln_x=@(x,y,z) nu1*x^(nu1-1)*y^nu2*z^nu3;
soln_y=@(x,y,z) nu2*x^nu1*y^(nu2-1)*z^nu3;
soln_z=@(x,y,z) nu3*x^nu1*y^(nu2)*z^(nu3-1);
soln_xx=@(x,y,z) nu1*(nu1-1)*x^(nu1-2)*y^nu2*z^nu3;
soln_yy=@(x,y,z) nu2*(nu2-1)*x^nu1*y^(nu2-2)*z^nu3;
soln_zz=@(x,y,z) nu3*(nu3-1)*x^nu1*y^nu2*z^(nu3-2);

L=@(x,y,z) 0.5*sig^2*(x^2*soln_xx(x,y,z)+y^2*soln_yy(x,y,z)+z^2*soln_zz(x,y,z)) ...
    +g*(x*soln_x(x,y,z)+y*soln_y(x,y,z)+z*soln_z(x,y,z))+r*soln(x,y,z);

utrue = X.^nu1.*Y.^nu2.*Z.^nu3;

usoln=zeros(m+2,m+2,m+2);
usoln(1,:,:)=a.^nu1.*y.^nu2'*z.^nu3;
usoln(m+2,:,:)=b.^nu1.*y.^nu2'*z.^nu3;
usoln(:,1,:)=x.^nu1'.*a.^nu2*z.^nu3;
usoln(:,m+2,:)=x.^nu1'.*b.^nu2*z.^nu3;
usoln(:,:,1)=x.^nu1'*y.^nu2.*a.^nu3;
usoln(:,:,m+2)=x.^nu1'*y.^nu2.*b.^nu3;

C=(.5*sig^2*(nu1*(nu1-1)+nu2*(nu2-1)+nu3*(nu3-1))+g*(nu1+nu2+nu3)+r);
f = @(x,y,z) C*x.^nu1.*y.^nu2.*z.^nu3;
rhs = f(Xint,Yint,Zint);

% rhs(:,1,:) = rhs(:,1,:) - 1/2*sig^2*Y(Iint,2).^2.*usoln(Iint,1)/h^2 + g*Y(Iint,2).*usoln(Iint,1)/(2*h);
% rhs(:,m,:) = rhs(:,m,:) - 1/2*sig^2*Y(Iint,m+1).^2.*usoln(Iint,m+2)/h^2 - g*Y(Iint,m+1).*usoln(Iint,m+2)/(2*h) ;
% rhs(1,:,:) = rhs(1,:,:) - 1/2*sig^2*X(2,Jint).^2.*usoln(1,Jint)/h^2 + g*X(2,Jint).*usoln(1,Jint)/(2*h);
% rhs(m,:,:) = rhs(m,:,:) - 1/2*sig^2*X(m+1,Jint).^2.*usoln(m+2,Jint)/h^2 - g*X(m+1,Jint).*usoln(m+2,Jint)/(2*h);

rhs(:,1,:) = rhs(:,1,:) - Cyy(:,1,:).*usoln(Iint,1,Kint)/h^2 + Cy(:,1,:).*usoln(Iint,1,Kint)/(2*h);
rhs(:,m,:) = rhs(:,m,:) - Cyy(:,m,:).*usoln(Iint,m+2,Kint)/h^2 - Cy(:,m,:).*usoln(Iint,m+2,Kint)/(2*h) ;
rhs(1,:,:) = rhs(1,:,:) - Cxx(1,:,:).*usoln(1,Jint,Kint)/h^2 + Cx(1,:,:).*usoln(1,Jint,Kint)/(2*h);
rhs(m,:,:) = rhs(m,:,:) - Cxx(m,:,:).*usoln(m+2,Jint,Kint)/h^2 - Cx(m,:,:).*usoln(m+2,Jint,Kint)/(2*h);
rhs(:,:,1) = rhs(:,:,1) - Czz(:,:,1).*usoln(Iint,Jint,1)/h^2 + Cz(:,:,1).*usoln(Iint,Jint,1)/(2*h);
rhs(:,:,m) = rhs(:,:,m) - Czz(:,:,m).*usoln(Iint,Jint,m+2)/h^2 - Cz(:,:,m).*usoln(Iint,Jint,m+2)/(2*h);

F = reshape(rhs,m*m*m,1);

Cxx=reshape(Cxx,m^3,1);
Cyy=reshape(Cyy,m^3,1);
Czz=reshape(Czz,m^3,1);
Cx=reshape(Cx,m^3,1);
Cy=reshape(Cy,m^3,1);
Cz=reshape(Cz,m^3,1);
C0=reshape(C0,m^3,1);

I = speye(m);
e = ones(m,1);
T = spdiags([e -2*e e],[-1 0 1],m,m);
B1 = spdiags(Cxx,0,m^3,m^3)*kron(I,kron(I,T)) ... 
    + spdiags(Cyy,0,m^3,m^3)*kron(I,kron(T,I)) ...
    + spdiags(Czz,0,m^3,m^3)*kron(T,speye(m*m));
B1=B1/h^2;
T2=spdiags([-e e],[-1 1],m,m);
B2=spdiags(Cx,0,m^3,m^3)*kron(I,kron(I,T2))...
    + spdiags(Cy,0,m^3,m^3)*kron(I,kron(T2,I))...
    + spdiags(Cz,0,m^3,m^3)*kron(T2,speye(m*m));
B2=B2/(2*h);
B3=spdiags(C0,0,m^3,m^3)*speye(m^3);
B=B1+B2+B3;

uvec =B\F;
usoln(Iint,Jint,Kint) = reshape(uvec,m,m,m);
err = max(max(max(abs(usoln-utrue))));   
fprintf('Error relative to true solution of PDE = %10.3e \n',err)

% usoln_int=utrue(Iint,Jint,Kint);
% usoln_vec=reshape(usoln_int,m^3,1);
% i=50;
% j=1;
% k=11;
% xx=Xint(i,j,k);
% yy=Yint(i,j,k);
% zz=Zint(i,j,k);
% num=(k-1)*m^2+(j-1)*m+i;
% soln(xx,yy,zz)
% usoln_vec(num)
% 
% % 0.5*sig^2*(xx^2*soln_xx(xx,yy,zz)...
% %     +yy^2*soln_yy(xx,yy,zz)...
% %     +zz^2*soln_zz(xx,yy,zz))
% % B1(num,:)*usoln_vec
% % 
% % g*(xx*soln_x(xx,yy,zz)+yy*soln_y(xx,yy,zz)+zz*soln_z(xx,yy,zz))
% % B2(num,:)*usoln_vec
% % 
% % r*soln(xx,yy,zz)
% % B3(num,:)*usoln_vec
% 
% %L(xx,yy,zz)
% B(num,:)*usoln_vec
% F(num)
% %f(xx,yy,zz)
% 
% test=B*usoln_vec-F;
% max(max(abs(test)))
% 
% 
% 
% 
