clear;
a = 0; 
b = 1; 
m =3;
h = (b-a)/(m+1);
x = linspace(a,b,m+2);   % grid points x including boundaries
y = linspace(a,b,m+2);   % grid points y including boundaries

[X,Y] = meshgrid(x,y);      % 2d arrays of x,y values
X = X';                     % transpose so that X(i,j),Y(i,j) are
Y = Y';                     % coordinates of (i,j) point

Iint = 2:m+1;              % indices of interior points in x
Jint = 2:m+1;              % indices of interior points in y
Xint = X(Iint,Jint);       % interior points
Yint = Y(Iint,Jint);

f = @(x,y) 2*exp(x+y/2);         % f(x,y) function

rhs = f(Xint,Yint);        % evaluate f at interior points for right hand side
                           % rhs is modified below for boundary conditions.

utrue = exp(X+Y/2);        % true solution for test problem

% set boundary conditions around edges of usoln array:

usoln = utrue;              % use true solution for this test problem
                            % This sets full array, but only boundary values
                            % are used below.  For a problem where utrue
                            % is not known, would have to set each edge of
                            % usoln to the desired Dirichlet boundary values.
                            
% adjust the rhs to include boundary terms:
rhs(:,1) = rhs(:,1) - usoln(Iint,1)/h^2 + usoln(Iint,1)/(2*h);
rhs(:,m) = rhs(:,m) - usoln(Iint,m+2)/h^2 - usoln(Iint,m+2)/(2*h) ;
rhs(1,:) = rhs(1,:) - usoln(1,Jint)/h^2 + usoln(1,Jint)/(2*h);
rhs(m,:) = rhs(m,:) - usoln(m+2,Jint)/h^2 - usoln(m+2,Jint)/(2*h);  

% convert the 2d grid function rhs into a column vector for rhs of system:
F = reshape(rhs,m*m,1);

I = speye(m);
e = ones(m,1);
T = spdiags([e -4*e e],[-1 0 1],m,m);
T2 = spdiags([-e e],[-1 1],m,m);
S = spdiags([e e],[-1 1],m,m);
S2 = spdiags([-e e],[-1 1],m,m);
A1 = (kron(I,T) + kron(S,I));
A2 = (kron(I,T2) + kron(S2,I));
A3 = kron(I,I);
A=A1/h^2+A2/(2*h)-.75*A3;

uvec = A\F; 

usoln(Iint,Jint) = reshape(uvec,m,m);

err = max(max(abs(usoln-utrue)));   
fprintf('Error relative to true solution of PDE = %10.3e \n',err)