clear;
a = 0.5; 
b = 2; 
m = 50;
h = (b-a)/(m+1);
x = linspace(a,b,m+2);   % grid points x including boundaries
y = linspace(a,b,m+2);   % grid points y including boundaries
tol=1e-7;

r=-0.05;
g=.02;
sig=.4;
nu1=.5;
nu2=.5;

[X,Y] = meshgrid(x,y);      % 2d arrays of x,y values
X = X';                     % transpose so that X(i,j),Y(i,j) are
Y = Y';                     % coordinates of (i,j) point

Iint = 2:m+1;              % indices of interior points in x
Jint = 2:m+1;              % indices of interior points in y
Xint = X(Iint,Jint);       % interior points
Yint = Y(Iint,Jint);

Cxx=reshape(0.5*sig^2*X(Iint,Jint).^2,m*m,1);
Cyy=reshape(0.5*sig^2*Y(Iint,Jint).^2,m*m,1);
Cx=reshape(g*X(Iint,Jint),m*m,1);
Cy=reshape(g*Y(Iint,Jint),m*m,1);
C0=reshape(r*ones(m,m),m*m,1);

utrue = X.^nu1.*Y.^nu2;  

soln=@(x,y) x^nu1*y^nu2;
soln_x=@(x,y) nu1*x^(nu1-1)*y^nu2;
soln_y=@(x,y) nu2*x^nu1*y^(nu2-1);
soln_xx=@(x,y) nu1*(nu1-1)*x^(nu1-2)*y^nu2;
soln_yy=@(x,y) nu2*(nu2-1)*x^nu1*y^(nu2-2);

usoln=zeros(m+2,m+2);
usoln(1,:)=a.^nu1.*y.^nu2;
usoln(m+2,:)=b.^nu1.*y.^nu2;
usoln(:,1)=x.^nu1.*a.^nu2;
usoln(:,m+2)=x.^nu1.*b.^nu2;

C=(.5*sig^2*(nu1*(nu1-1)+nu2*(nu2-1))+g*(nu1+nu2)+r);
f = @(x,y) C*x.^nu1.*y.^nu2;
rhs = f(Xint,Yint);  

rhs(:,1) = rhs(:,1) - 1/2*sig^2*Y(Iint,2).^2.*usoln(Iint,1)/h^2 + g*Y(Iint,2).*usoln(Iint,1)/(2*h);
rhs(:,m) = rhs(:,m) - 1/2*sig^2*Y(Iint,m+1).^2.*usoln(Iint,m+2)/h^2 - g*Y(Iint,m+1).*usoln(Iint,m+2)/(2*h) ;
rhs(1,:) = rhs(1,:) - 1/2*sig^2*X(2,Jint).^2.*usoln(1,Jint)/h^2 + g*X(2,Jint).*usoln(1,Jint)/(2*h);
rhs(m,:) = rhs(m,:) - 1/2*sig^2*X(m+1,Jint).^2.*usoln(m+2,Jint)/h^2 - g*X(m+1,Jint).*usoln(m+2,Jint)/(2*h);

F = reshape(rhs,m*m,1);

% I = speye(m);
% e = ones(m,1);
% T = spdiags([e -2*e e],[-1 0 1],m,m);
% A1 = kron(I,.5*sig^2*diag(x(2:m+1).^2)*T) ...
%     +kron(.5*sig^2*diag(y(2:m+1).^2)*T,I);
% A1=A1/h^2;
% T2=spdiags([-e e],[-1 1],m,m);
% A2=kron(I,g*diag(x(2:m+1))*T2) ...
%     +kron(g*diag(y(2:m+1))*T2,I);
% A2=A2/(2*h);
% A3=r*kron(I,I);
% A=A1+A2+A3;
% A=full(A);

I = speye(m);
e = ones(m,1);
T = spdiags([e -2*e e],[-1 0 1],m,m);
B1 = spdiags(Cxx,0,m*m,m*m)*kron(I,T) ... 
    + spdiags(Cyy,0,m*m,m*m)*kron(T,I);
B1=B1/h^2;
T2=spdiags([-e e],[-1 1],m,m);
B2=spdiags(Cx,0,m*m,m*m)*kron(I,T2) + spdiags(Cy,0,m*m,m*m)*kron(T2,I);
B2=B2/(2*h);
B3=spdiags(C0,0,m*m,m*m)*kron(I,I);
B=B1+B2+B3;

%uvec = A\F; 
uvec =B\F;
usoln(Iint,Jint) = reshape(uvec,m,m);
err = max(max(abs(usoln-utrue)));   
fprintf('Error relative to true solution of PDE = %10.3e \n',err)

% usoln=utrue;
% usoln_vec=reshape(usoln(Iint,Jint),m*m,1);
% test=A*usoln_vec-F;
% val=max(max(abs(test)))

% num=2;
% xx=x(2+mod(num-1,m));
% yy=y(2+floor((num-.01)/m));
% soln(xx,yy)
% usoln_vec(num)
% 
% % A1(num,:)*usoln_vec
% % 1/2*sig^2*(xx^2*soln_xx(xx,yy)+yy^2*soln_yy(xx,yy))
% % A2(num,:)*usoln_vec
% % g*(xx*soln_x(xx,yy)+yy*soln_y(xx,yy))
% % A3(num,:)*usoln_vec
% % r*soln(xx,yy)
% 
% A(num,:)*usoln_vec
% 1/2*sig^2*(xx^2*soln_xx(xx,yy)+yy^2*soln_yy(xx,yy)) ...
%     +g*(xx*soln_x(xx,yy)+yy*soln_y(xx,yy)) ...
%     +r*soln(xx,yy)
% f(xx,yy)
% F(num)

% num=21;
% xx=x(2+mod(num-1,m));
% yy=y(2+floor((num-.01)/m));
% soln(xx,yy)
% usoln_vec(num)
% 
% A(num,:)*usoln_vec
% F(num)
