clear;
a = 0.5; 
b = 2; 
m = 50;
h = (b-a)/(m+1);
x = linspace(a,b,m+2);   % grid points x including boundaries
y = linspace(a,b,m+2);   % grid points y including boundaries
tol=1e-8;

r=-0.05;
g=.02;
sig=.4;
nu1=.5;
nu2=.5;

[X,Y] = meshgrid(x,y);      % 2d arrays of x,y values
X = X';                     % transpose so that X(i,j),Y(i,j) are
Y = Y';                     % coordinates of (i,j) point

Iint = 2:m+1;              % indices of interior points in x
Jint = 2:m+1;              % indices of interior points in y
Xint = X(Iint,Jint);       % interior points
Yint = Y(Iint,Jint);

utrue = X.^nu1.*Y.^nu2;  

soln_x=@(x,y) nu1*x^(nu1-1)*y^nu2;
soln_y=@(x,y) nu2*x^nu1*y^(nu2-1);
soln_xx=@(x,y) nu1*(nu1-1)*x^(nu1-2)*y^nu2;
soln_yy=@(x,y) nu2*(nu2-1)*x^nu1*y^(nu2-2);

usoln=zeros(m+2,m+2);
usoln(1,:)=a.^nu1.*y.^nu2;
usoln(m+2,:)=b.^nu1.*y.^nu2;
usoln(:,1)=x.^nu1.*a.^nu2;
usoln(:,m+2)=x.^nu1.*b.^nu2;

C=(.5*sig^2*(nu1*(nu1-1)+nu2*(nu2-1))+g*(nu1+nu2)+r);
f = @(x,y) C*x.^nu1.*y.^nu2;
source = f(Xint,Yint);

iter=1;
Div=(1/2*sig^2*X(Iint,Jint).^2.*2/h^2+1/2*sig^2*Y(Iint,Jint).^2.*2/h^2-r);
test=usoln(Iint,Jint)-1./Div.*(1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)+usoln(Iint-1,Jint))/h^2 ...
    +1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)+usoln(Iint,Jint-1))/h^2 ...
    +g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h) ...
    +g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h) ...
    -source);

val=max(max(abs(test)));
usoln_old=usoln;
while val>tol 
usoln(Iint,Jint)=1./Div.*(1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)+usoln(Iint-1,Jint))/h^2 ...
    +1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)+usoln(Iint,Jint-1))/h^2 ...
    +g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h) ...
    +g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h) ...
    -source);
test=usoln(Iint,Jint)-1./Div.*(1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)+usoln(Iint-1,Jint))/h^2 ...
    +1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)+usoln(Iint,Jint-1))/h^2 ...
    +g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h) ...
    +g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h) ...
    -source);
val=max(max(abs(test)));
usoln_old=usoln;
iter=iter+1;
end

err = max(max(abs(usoln-utrue)));   
fprintf('Error relative to true solution of PDE = %10.3e \n',err)

surf(X,Y,usoln)
surf(X,Y,usoln-utrue)
    
% i=70;
% j=10;
% ii=i-1;
% jj=j-1;
% x=X(i,j);
% y=Y(i,j);
% 
% uxx=1/2*sig^2*X(i,j).^2.*(usoln(i+1,j)-2*usoln(i,j)+usoln(i-1,j))/h^2;
% uyy=1/2*sig^2*Y(i,j).^2.*(usoln(i,j+1)-2*usoln(i,j)+usoln(i,j-1))/h^2;
% ux=g*X(i,j).*(usoln(i+1,j)-usoln(i-1,j))/(2*h);
% uy=g*Y(i,j).*(usoln(i,j+1)-usoln(i,j-1))/(2*h);
% u=usoln(i,j);
% uxx+uyy+ux+uy+r*u-f(x,y)
% 
% sxx=1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)-2*usoln(Iint,Jint)+usoln(Iint-1,Jint))/h^2;
% syy=1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)-2*usoln(Iint,Jint)+usoln(Iint,Jint-1))/h^2;
% sx=g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h);
% sy=g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h);
% su=usoln(Iint,Jint);
% test2=sxx+syy+sx+sy+r*su-source;
% 
% test3=1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)-2*usoln(Iint,Jint)+usoln(Iint-1,Jint))/h^2 ...
%     +1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)-2*usoln(Iint,Jint)+usoln(Iint,Jint-1))/h^2 ...
%     +g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h) ...
%     +g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h) ...
%     +r*usoln(Iint,Jint)-source;
% val=max(max(abs(test3)))
% 
% Div=(1/2*sig^2*X(Iint,Jint).^2.*2/h^2+1/2*sig^2*Y(Iint,Jint).^2.*2/h^2-r);
% test4=Div.*usoln(Iint,Jint)-(1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)+usoln(Iint-1,Jint))/h^2 ...
%     +1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)+usoln(Iint,Jint-1))/h^2 ...
%     +g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h) ...
%     +g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h) ...
%     -source);
% 
% test5=usoln(Iint,Jint)-1./Div.*(1/2*sig^2*X(Iint,Jint).^2.*(usoln(Iint+1,Jint)+usoln(Iint-1,Jint))/h^2 ...
%     +1/2*sig^2*Y(Iint,Jint).^2.*(usoln(Iint,Jint+1)+usoln(Iint,Jint-1))/h^2 ...
%     +g*X(Iint,Jint).*(usoln(Iint+1,Jint)-usoln(Iint-1,Jint))/(2*h) ...
%     +g*Y(Iint,Jint).*(usoln(Iint,Jint+1)-usoln(Iint,Jint-1))/(2*h) ...
%     -source);
% 
% 
% 
%     
%     