close all;
clear;
r=0.5;
model=createpde;

%R1 = [3,4,-1,1,1,-1,0.5,0.5,-0.75,-0.75]';
R1 = [3,4,0,1,1,0,0,0,1,1]';
gm = R1;
sf = 'R1';
ns = char('R1');
ns = ns';
g = decsg(gm,sf,ns);
geometryFromEdges(model,g);
pdegplot(model,'EdgeLabels','on','FaceLabels','on')
axis equal
xlim([-.1,1.1])

BC=@(location,state) exp(location.x+location.y/2);
Source=@(location,state) (1.25+r)*exp(location.x+location.y/2);
applyBoundaryCondition(model,'dirichlet','Edge',1:model.Geometry.NumEdges,'u',BC);
specifyCoefficients(model,'m',0,'d',0,'c',-1,'a',r,'f',Source);
coeffs=model.EquationCoefficients;
BCs = model.BoundaryConditions;

hmax = 0.1;
generateMesh(model,'Hmax',hmax);
figure
pdemesh(model); 
axis equal

results = solvepde(model);
u = results.NodalSolution;
pdeplot(model,'XYData',u)
title('Numerical Solution');
xlabel('x')
ylabel('y')

p = model.Mesh.Nodes;
exact = exp(p(1,:)+p(2,:)/2);
pdeplot(model,'XYData',u - exact')
title('Error');
xlabel('x')
ylabel('y')

hmax = 0.1;
error = [];
err = 1;
while err > 5e-7 % run until error <= 5e-7
    generateMesh(model,'Hmax',hmax); % refine mesh
    results = solvepde(model);
    u = results.NodalSolution;
    p = model.Mesh.Nodes;
    exact = exp(p(1,:)+p(2,:)/2);
    err = norm(u - exact',inf); % compare with exact solution
    error = [error err]; % keep history of err
    hmax = hmax/2;
end

plot(error,'rx','MarkerSize',12);
ax = gca;
ax.XTick = 1:numel(error);
title('Error History');
xlabel('Iteration');
ylabel('Norm of Error');

figure
pdemesh(model); 
axis equal

figure
pdeplot(model,'XYData',u)
title('Numerical Solution');
xlabel('x')
ylabel('y')

p = model.Mesh.Nodes;
exact = exp(p(1,:)+p(2,:)/2);
pdeplot(model,'XYData',u - exact')
title('Error');
xlabel('x')
ylabel('y')